@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library (in the file LGPL.txt); if not, 
+ * write to the Free Software Foundation, Inc., 51 Franklin St, 
+ * Fifth Floor, Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -31,6 +43,7 @@
 #include <map>
 
 // svncpp
+#include "svncpp/check.hpp"
 #include "svncpp/context.hpp"
 #include "svncpp/exception.hpp"
 #include "svncpp/path.hpp"
@@ -73,7 +86,6 @@ namespace svn
 
     virtual ~Client ();
 
-
     /**
      * @return returns the Client context
      */
@@ -100,6 +112,7 @@ namespace svn
      * @param get_all Return all entries, not just the interesting ones.
      * @param update Query the repository for updates.
      * @param no_ignore Disregard default and svn:ignore property ignores.
+     * @param ignore_externals Disregard external files.
      * @return vector with Status entries.
      */
     StatusEntries 
@@ -107,7 +120,8 @@ namespace svn
             const bool descend = false,
             const bool get_all = true,
             const bool update = false,
-            const bool no_ignore = false) throw (ClientException);
+            const bool no_ignore = false,
+            const bool ignore_externals = false) throw (ClientException);
 
     /**
      * Returns the status of a single file in the path.
@@ -120,19 +134,24 @@ namespace svn
     Status 
     singleStatus (const char * path) throw (ClientException);
 
-  /**
+    /**
      * Executes a revision checkout.
      * @param moduleName name of the module to checkout.
      * @param destPath destination directory for checkout.
      * @param revision the revision number to checkout. If the number is -1
      *                 then it will checkout the latest revision.
      * @param recurse whether you want it to checkout files recursively.
+     * @param ignore_externals whether you want get external resources too.
+     * @param peg_revision peg revision to checkout, by default current.
      * @exception ClientException
      */
     svn_revnum_t
-    checkout (const char * moduleName, const Path & destPath, 
+    checkout (const char * moduleName,
+              const Path & destPath, 
               const Revision & revision, 
-              bool recurse) throw (ClientException);
+              bool recurse,
+              bool ignore_externals = false,
+              const Revision & peg_revision = Revision ()) throw (ClientException);
   
     /**
      * relocate wc @a from to @a to
@@ -160,6 +179,28 @@ namespace svn
     remove (const Targets & targets, 
             bool force) throw (ClientException);
 
+    /**
+     * Sets files to lock.
+     *
+     * @param targets targets to lock
+     * @param force force setting/stealing lock
+     * @param comment writing comment about lock setting is neccessary
+     * @exception ClientException
+     */
+    void 
+    lock (const Targets & targets, bool force,
+          const char * comment) throw (ClientException);
+
+    /**
+     * Sets files to unlock.
+     *
+     * @param targets targets to unlock
+     * @param force force unlock even if lock belongs to another user
+     * @exception ClientException
+     */
+    void 
+    unlock (const Targets & targets, bool force) throw (ClientException);
+
     /**
      * Reverts a couple of files to a pristiner state.
      * @exception ClientException
@@ -167,7 +208,6 @@ namespace svn
     void
     revert (const Targets & targets, bool recurse) throw (ClientException);
 
-
     /**
      * Adds a file to the repository.
      * @exception ClientException
@@ -183,11 +223,27 @@ namespace svn
      *                 latest revision.
      * @param recurse recursively update.
      * @exception ClientException
+     *
+     * @deprecated
      */
     svn_revnum_t 
     update (const Path & path, const Revision & revision, 
             bool recurse) throw (ClientException);
 
+    /**
+     * Updates the file or directory.
+     * @param Targets target files.
+     * @param revision the revision number to checkout. 
+     *                 Revision::HEAD will checkout the 
+     *                 latest revision.
+     * @param recurse recursively update.
+     * @param ignore_externals don't affect external destinations.
+     * @exception ClientException
+     */
+    void
+    update2 (const Targets & targets, const Revision & revision, 
+             bool recurse, bool ignore_externals) throw (ClientException);
+
     /**
      * Retrieves the contents for a specific @a revision of
      * a @a path
@@ -198,7 +254,8 @@ namespace svn
      */
     std::string
     cat (const Path & path, 
-         const Revision & revision) throw (ClientException);
+         const Revision & revision,
+         const Revision & peg_revision = Revision ()) throw (ClientException);
 
 
     /**
@@ -246,9 +303,10 @@ namespace svn
      * @exception ClientException
      */
     svn_revnum_t
-    commit (const Targets & targets, 
+    commit (const Targets & targets,
             const char * message, 
-            bool recurse) throw (ClientException);
+            bool recurse,
+            bool keep_locks = false) throw (ClientException);
 
     /**
      * Copies a versioned file with the history preserved.
@@ -315,7 +373,30 @@ namespace svn
     doExport (const Path & srcPath, 
               const Path & destPath, 
               const Revision & revision, 
-              bool force=false) throw (ClientException);
+              bool force = false) throw (ClientException);
+
+    /**
+     * Export into file or directory TO_PATH from local or remote FROM_PATH
+     * @param from_path path to import
+     * @param to_path where to import
+     * @param revision revision of files in source repository or working copy
+     * @param peg_revision
+     * @param overwrite overwrite existing files in to_path
+     * @param ignore_externals whether to ignore external sources in from_path
+     * @param recurse
+     * @param native_eol which EOL to use when exporting, usually different for
+     * different OSs
+     * @exception ClientException
+     */
+    void
+    doExport2 (const Path & from_path,
+               const Path & to_path,
+               const Revision & revision,
+               bool overwrite = false,
+               const Revision & peg_revision = Revision (),
+               bool ignore_externals = false,
+               bool recurse = true,
+               const char * native_eol = NULL) throw (ClientException);
 
     /**
      * Update local copy to mirror a new url. This excapsulates the 
@@ -337,10 +418,12 @@ namespace svn
      * @exception ClientException
      */
     void 
-    import (const Path & path, const char * url,
+    import (const Path & path,
+            const char * url,
             const char * message, 
             bool recurse) throw (ClientException);
 
+
     /**
      * Merge changes from two paths into a new local path.
      * @exception ClientException
@@ -350,8 +433,8 @@ namespace svn
            const Path & path2, const Revision & revision2,
            const Path & localPath, bool force, 
            bool recurse,
-           bool notice_ancestry=false,
-           bool dry_run=false) throw (ClientException);
+           bool notice_ancestry = false,
+           bool dry_run = false) throw (ClientException);
 
     /**
      * Retrieve information for the given path
@@ -361,7 +444,7 @@ namespace svn
      * @return Entry
      */
     Entry
-    info (const char *path );
+    info (const char * path);
 
     /**
      * Retrieve log information for the given path
@@ -379,10 +462,11 @@ namespace svn
      * @return a vector with log entries
      */
     const LogEntries *
-    log (const char * path, const Revision & revisionStart, 
+    log (const char * path,
+         const Revision & revisionStart, 
          const Revision & revisionEnd,
-         bool discoverChangedPaths=false, 
-         bool strictNodeHistory=true) throw (ClientException);
+         bool discoverChangedPaths = false, 
+         bool strictNodeHistory = true) throw (ClientException);
 
     /**
      * Produce diff output which describes the delta between
@@ -465,9 +549,9 @@ namespace svn
      * @return PropertiesList
      */
     PathPropertiesMapList
-    proplist(const Path &path,
-             const Revision &revision,
-             bool recurse=false);
+    proplist (const Path &path,
+              const Revision &revision,
+              bool recurse = false);
 
     /**
      * lists one property in @a path no matter whether local or
@@ -480,28 +564,34 @@ namespace svn
      * @return PathPropertiesMapList
      */
     PathPropertiesMapList
-    propget(const char *propName,
-            const Path &path,
-            const Revision &revision,
-            bool recurse=false);
+    propget (const char * propName,
+             const Path & path,
+             const Revision & revision,
+             bool recurse = false);
 
     /**
+     * This method is deprecated, please use
+     * @a Property.set
      * set property in @a path no matter whether local or
      * repository
      *
+     * @deprecated
+     *
      * @param path
      * @param revision
      * @param propName
      * @param propValue
      * @param recurse
+     * @param skip_checks
      * @return PropertiesList
      */
     void
-    propset(const char *propName,
-            const char *propValue,
-            const Path &path,
-            const Revision &revision,
-            bool recurse=false);
+    propset (const char * propName,
+             const char * propValue,
+             const Path & path,
+             const Revision & revision,
+             bool recurse = false,
+             bool skip_checks = true);
 
     /**
      * delete property in @a path no matter whether local or
@@ -513,10 +603,10 @@ namespace svn
      * @param recurse
      */
     void
-    propdel(const char *propName,
-            const Path &path,
-            const Revision &revision,
-            bool recurse=false);
+    propdel (const char * propName,
+             const Path & path,
+             const Revision & revision,
+             bool recurse = false);
 
 
     /**
@@ -528,8 +618,8 @@ namespace svn
      * @return PropertiesList
      */
     std::pair<svn_revnum_t,PropertiesMap>
-    revproplist(const Path &path,
-                const Revision &revision);
+    revproplist (const Path & path,
+                 const Revision & revision);
 
     /**
      * lists one revision property in @a path no matter whether local or
@@ -541,9 +631,9 @@ namespace svn
      * @return PropertiesList
      */
     std::pair<svn_revnum_t,std::string>
-    revpropget(const char *propName,
-               const Path &path,
-               const Revision &revision);
+    revpropget (const char * propName,
+                const Path & path,
+                const Revision & revision);
 
     /**
      * set revision property in @a path no matter whether local or
@@ -557,11 +647,11 @@ namespace svn
      * @return Revision
      */
     svn_revnum_t
-    revpropset(const char *propName,
-               const char *propValue,
-               const Path &path,
-               const Revision &revision,
-               bool force=false);
+    revpropset (const char * propName,
+                const char * propValue,
+                const Path & path,
+                const Revision & revision,
+                bool force = false);
 
     /**
      * delete revision property in @a path no matter whether local or
@@ -574,10 +664,10 @@ namespace svn
      * @return Revision
      */
     svn_revnum_t
-    revpropdel(const char *propName,
-               const Path &path,
-               const Revision &revision,
-               bool force=false);
+    revpropdel (const char * propName,
+                const Path & path,
+                const Revision & revision,
+                bool force = false);
 
 
   private:
@@ -592,9 +682,7 @@ namespace svn
      * disallow copy constructor
      */
     Client (const Client &);
-
   };
-
 }
 
 #endif
