@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -17,6 +29,7 @@
 
 // svncpp
 #include "svncpp/client.hpp"
+// #include "svncpp/m_check.hpp"
 
 // app
 #include "action.hpp"
@@ -30,31 +43,42 @@
 
 //TODO: This isn't actually used, and an exact copy lives in listener.cpp which is used
 // I suspect we should get rid of one, but I'm not sure which at the  moment...
-static const wxChar *
-ACTION_NAMES [] =
-{
-  _("Add"),           // svn_wc_notify_add,
-  _("Copy"),          // svn_wc_notify_copy,
-  _("Delete"),        // svn_wc_notify_delete,
-  _("Restore"),       // svn_wc_notify_restore,
-  _("Revert"),        // svn_wc_notify_revert,
-  NULL ,              // NOT USED HERE svn_wc_notify_failed_revert,
-  _("Resolved"),      // svn_wc_notify_resolved,
-  _("Skip"),          // NOT USED HERE svn_wc_notify_skip,
-  _("Deleted"),       // svn_wc_notify_update_delete,
-  _("Added"),         // svn_wc_notify_update_add,
-  _("Updated"),       // svn_wc_notify_update_update,
-  NULL,               // NOT USED HERE svn_wc_notify_update_completed,
-  NULL,               // NOT USED HERE svn_wc_notify_update_external,
-  NULL,               // NOT USED HERE svn_wc_notify_status_completed
-  NULL,               // NOT USED HERE svn_wc_notify_status_external
-  _("Modified"),      // svn_wc_notify_commit_modified,
-  _("Added"),         // svn_wc_notify_commit_added,
-  _("Deleted"),       // svn_wc_notify_commit_deleted,
-  _("Replaced"),      // svn_wc_notify_commit_replaced,
-  NULL                // NOT USED HERE svn_wc_notify_commit_postfix_txdelta
-};
-const int MAX_ACTION = svn_wc_notify_commit_postfix_txdelta;
+// static const wxChar *
+// ACTION_NAMES [] =
+// {
+//   _("Add"),              // svn_wc_notify_add,
+//   _("Copy"),             // svn_wc_notify_copy,
+//   _("Delete"),           // svn_wc_notify_delete,
+//   _("Restore"),          // svn_wc_notify_restore,
+//   _("Revert"),           // svn_wc_notify_revert,
+//   NULL ,                 // NOT USED HERE svn_wc_notify_failed_revert,
+//   _("Resolved"),         // svn_wc_notify_resolved,
+//   _("Skip"),             // NOT USED HERE svn_wc_notify_skip,
+//   _("Deleted"),          // svn_wc_notify_update_delete,
+//   _("Added"),            // svn_wc_notify_update_add,
+//   _("Updated"),          // svn_wc_notify_update_update,
+//   NULL,                  // NOT USED HERE svn_wc_notify_update_completed,
+//   NULL,                  // NOT USED HERE svn_wc_notify_update_external,
+//   NULL,                  // NOT USED HERE svn_wc_notify_status_completed
+//   NULL,                  // NOT USED HERE svn_wc_notify_status_external#include "svncpp/m_check.hpp"
+
+//   _("Modified"),         // svn_wc_notify_commit_modified,
+//   _("Added"),            // svn_wc_notify_commit_added,
+//   _("Deleted"),          // svn_wc_notify_commit_deleted,
+//   _("Replaced"),         // svn_wc_notify_commit_replaced,
+//   NULL,                  // NOT USED HERE svn_wc_notify_commit_postfix_txdelta
+//   NULL,                  // NOT USED HERE svn_wc_notify_blame_revision
+//   _("Locked"),           // svn_wc_notify_locked
+//   _("Unlocked"),         // svn_wc_notify_unlocked
+//   _("Failed to lock"),   // svn_wc_notify_failed_lock
+//   _("Failed to unlock")  // svn_wc_notify_failed_unlock
+// };
+
+// #if CHECK_SVN_SUPPORTS_LOCK
+// const int MAX_ACTION = svn_wc_notify_failed_unlock;
+// #else
+// const int MAX_ACTION = svn_wc_notify_commit_postfix_txdelta;
+// #endif
 
 #ifdef USE_SIMPLE_WORKER
 void
@@ -350,8 +374,10 @@ Action::GetPathAsTempFile (const svn::Path & path,
   wxString revStr;
   if (revision.kind () == revision.HEAD)
     revStr = _("HEAD");
+  else if (revision.kind() == revision.BASE)
+    revStr = _("BASE");
   else
-    revStr.Printf (wxT("%" SVN_REVNUM_T_FMT), revision.revnum ());
+    revStr.Printf (wxT("%") wxT(SVN_REVNUM_T_FMT), revision.revnum ());
 
   wxString msg, wxpath (Utf8ToLocal (path.c_str ()));
   msg.Printf (_("Get file %s rev. %s"),
