@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -67,20 +79,6 @@ public:
   }
 
 
-  /**
-   * retrieves a file @a path that has the same
-   * revision as the working copy
-   */
-  svn::Path
-  getFile (const svn::Path & path, 
-           const svn::Status & status)
-  {
-    svn::Revision revision (getRevision (path, status));
-
-    return mAction->GetPathAsTempFile (path, revision);
-  }
-
-
   /**
    * retrieves the status information for a file
    */
@@ -112,10 +110,7 @@ public:
   getPath1 (const svn::Path & path)
   {
     if (diffData.useUrl1)
-    {
-      std::string url1Utf8 (LocalToUtf8 (diffData.url1));
-      return svn::Path (url1Utf8);
-    }
+      return PathUtf8 (diffData.url1);
     return path;
   }
 
@@ -127,10 +122,7 @@ public:
   getPath2 (const svn::Path & path)
   {
     if (diffData.useUrl2)
-    {
-      std::string url2Utf8 (LocalToUtf8 (diffData.url1));
-      return svn::Path (url2Utf8);
-    }
+      return PathUtf8 (diffData.url1);
     else
       return path;
   }
@@ -184,9 +176,15 @@ public:
 
     switch (diffData.compareType)
     {
-    case DiffData::WITH_SAME_REVISION:
+    case DiffData::WITH_BASE:
       dstFile1 = path;
-      dstFile2 = getFile (path, status);
+      dstFile2 = mAction->GetPathAsTempFile (getPath1 (path), svn::Revision::BASE);
+
+      break;
+
+    case DiffData::WITH_HEAD:
+      dstFile1 = path;
+      dstFile2 = mAction->GetPathAsTempFile (getPath1 (path), svn::Revision::HEAD);
 
       break;
 
