@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -51,10 +63,11 @@ struct FileInfo::Data
   svn::Context * context;
   std::vector<svn::Path> targets;
   wxString info;
+  bool withUpdate;
 
 
-  Data (svn::Context * ctx)
-    : context (ctx)
+  Data (svn::Context * ctx, bool update)
+    : context (ctx), withUpdate (update)
   {
   }
 
@@ -86,7 +99,7 @@ struct FileInfo::Data
     addLine (str);
 
     tmp = Utf8ToLocal (entry.url ());
-    str.Printf (_("Url: %s"), tmp.c_str ());
+    str.Printf (_("URL: %s"), tmp.c_str ());
     addLine (str);
 
     if (entry.repos ()) {
@@ -159,7 +172,7 @@ struct FileInfo::Data
     if (entry.isCopied ())
     {
       tmp = Utf8ToLocal (entry.copyfromUrl ());
-      str.Printf (_("Copied From Url: %s"), tmp.c_str ());
+      str.Printf (_("Copied From URL: %s"), tmp.c_str ());
       addLine (str);
 
       str.Printf (_("Copied From Rev: %ld"), entry.copyfromRev ());
@@ -193,17 +206,17 @@ struct FileInfo::Data
     if (text_conflict)
     {
       tmp = Utf8ToLocal (entry.conflictOld ());
-      str.Printf (_("Conflict Previous Base File: %s"),
+      str.Printf (_("Conflict BASE File: %s"),
                   tmp.c_str ());
       addLine (str);
 
       tmp = Utf8ToLocal (entry.conflictWrk ());
-      str.Printf (_("Conflict Previous Working File: %s"),
+      str.Printf (_("Conflict Working File: %s"),
                   tmp.c_str ());
       addLine (str);
 
       tmp = Utf8ToLocal (entry.conflictNew ());
-      str.Printf (_("Conflict Current Base File: %s"),
+      str.Printf (_("Conflict HEAD File: %s"),
                   tmp.c_str ());
       addLine (str);
     }
@@ -215,6 +228,26 @@ struct FileInfo::Data
                   tmp.c_str ());
       addLine (str);
     }
+
+    if (status.isLocked ())
+    {
+      tmp = Utf8ToLocal (status.lockToken ());
+      str.Printf (_("Lock Token: %s"),
+                  tmp.c_str ());
+      addLine (str);
+
+      tmp = Utf8ToLocal (status.lockOwner ());
+      str.Printf (_("Lock Owner: %s"),
+                  tmp.c_str ());
+      addLine (str);
+
+      tmp = Utf8ToLocal (status.lockComment ());
+      str.Printf (_("Lock Comment:\n%s"),
+                  tmp.c_str ());
+      addLine (str);
+
+      info_print_time (status.lockCreationDate (), _("Lock Creation Date"), str);
+    }
   }
 
   void
@@ -222,7 +255,7 @@ struct FileInfo::Data
   {
     try
     {
-      svn::StatusEntries ent = client.status (path);
+      svn::StatusEntries ent = client.status (path, false, true, withUpdate);
       svn::StatusEntries::const_iterator it;
 
       for (it=ent.begin (); it!=ent.end (); it++)
@@ -242,9 +275,9 @@ struct FileInfo::Data
   }
 };
 
-FileInfo::FileInfo (svn::Context * context)
+FileInfo::FileInfo (svn::Context * context, bool withUpdate)
 {
-  m = new Data (context);
+  m = new Data (context, withUpdate);
 }
 
 FileInfo::~FileInfo ()
@@ -269,7 +302,6 @@ FileInfo::info () const
   for (it = m->targets.begin (); it != m->targets.end (); it++)
   {
     svn::Path path = *it;
-    svn::Status status = client.singleStatus (path.c_str ());
 
     m->createInfoForPath (client, path.c_str ());
     m->addLine ();
