@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -13,9 +25,9 @@
 #if defined( _MSC_VER) && _MSC_VER <= 1200
 #pragma warning( disable: 4786 )// debug symbol truncated
 #endif
-
 // stl
 #include <exception>
+#include <map>
 
 // wxWidgets
 #include "wx/wx.h"
@@ -57,34 +69,28 @@
 #include "res/bitmaps/newer_file.xpm"
 #include "res/bitmaps/newer_folder.xpm"
 #include "res/bitmaps/modified_newer_file.xpm"
+#include "res/bitmaps/externals_folder.xpm"
 
+#include "res/bitmaps/locked_normal_file.xpm"
+#include "res/bitmaps/locked_missing_file.xpm"
+#include "res/bitmaps/locked_deleted_file.xpm"
+#include "res/bitmaps/locked_replaced_file.xpm"
+#include "res/bitmaps/locked_modified_file.xpm"
+#include "res/bitmaps/locked_merged_file.xpm"
+#include "res/bitmaps/locked_conflicted_file.xpm"
+#include "res/bitmaps/locked_newer_file.xpm"
+#include "res/bitmaps/locked_modified_newer_file.xpm"
 
-/**
- * The index from where there will be only images not related
- * to the status.
- */
-static const int START_EXTRA_IMGS=15;
+#include "res/bitmaps/replock_normal_file.xpm"
+#include "res/bitmaps/replock_missing_file.xpm"
+#include "res/bitmaps/replock_deleted_file.xpm"
+#include "res/bitmaps/replock_replaced_file.xpm"
+#include "res/bitmaps/replock_modified_file.xpm"
+#include "res/bitmaps/replock_merged_file.xpm"
+#include "res/bitmaps/replock_conflicted_file.xpm"
+#include "res/bitmaps/replock_newer_file.xpm"
+#include "res/bitmaps/replock_modified_newer_file.xpm"
 
-enum
-{
-  IMG_INDX_FOLDER = START_EXTRA_IMGS,
-  IMG_INDX_VERSIONED_FOLDER,
-  IMG_INDX_SORT_DOWN,
-  IMG_INDX_SORT_UP,
-  IMG_INDX_MODIFIED_VERSIONED_FOLDER,
-  IMG_INDX_NEWER_FILE,
-  IMG_INDX_NEWER_FOLDER,
-  IMG_INDX_MODIFIED_NEWER,
-  IMG_INDX_COUNT
-};
-
-/**
- * This table holds information about image index in a image list.
- * It will be accessed using a status code as an index.
- * For every index that equals a valid status code, there should be
- * a valid value which represents an index in the image list.
- */
-static int IMAGE_INDEX[IMG_INDX_COUNT];
 
 /**
  * Tags for wxConfig file settings, defined here to avoid duplicate
@@ -97,6 +103,7 @@ static const wxChar ConfigColumnVisibleFmt[] = wxT("/FileListCtrl/Column%sVisibl
 static const wxChar ConfigWithUpdate[]       = wxT("/FileListCtrl/WithUpdate");
 static const wxChar ConfigFlatView[]         = wxT("/FileListCtrl/FlatView");
 static const wxChar ConfigShowUnversioned[]  = wxT("/FileListCtrl/ShowUnversioned");
+static const wxChar ConfigIgnoreExternals[]  = wxT("/FileListCtrl/IgnoreExternals");
 
 /**
  * test if the given status entry is a file or
@@ -119,17 +126,12 @@ IsDir (const svn::Status * status)
     return false;
   }
 
-  svn_wc_status_t * svn_status = *status;
-  if (!svn_status)
-  {
-    return false;
-  }
-
   // versioned resource?
-  svn_wc_entry_t * entry = svn_status->entry;
-  if (entry)
+  const svn::Entry & entry = status->entry ();
+
+  if (entry.isValid ())
   {
-    return entry->kind == svn_node_dir;
+    return entry.kind () == svn_node_dir;
   }
 
   // unversioned.
@@ -502,67 +504,104 @@ CompareItems (svn::Status * ps1, svn::Status * ps2,
 
 
 /**
- * array with column captions
+ * The index from where there will be only images
+ * not related to the status.
+ * See @a svn_wc_status_kind in @a svn_wc.h for more details.
  */
-static const wxChar *
-COLUMN_CAPTIONS[FileListCtrl::COL_COUNT] =
+enum
 {
-  _("Name"),
-  _("Path"),
-  _("Revision"),
-  _("Rep. Rev."),
-  _("Author"),
-  _("Status"),
-  _("Prop Status"),
-  _("Last Changed"),
-  _("Extension"),
-  _("Date"),
-  _("Prop Date"),
-  _("Checksum"),
-  _("Url"),
-  _("Repository"),
-  _("UUID"),
-  _("Schedule"),
-  _("Copied"),
-  _("Conflict Old"),
-  _("Conflict New"),
-  _("Conflict Work")
+  IMG_INDX_FOLDER = 15,
+  IMG_INDX_VERSIONED_FOLDER,
+  IMG_INDX_SORT_DOWN,
+  IMG_INDX_SORT_UP,
+  IMG_INDX_MODIFIED_VERSIONED_FOLDER,
+  IMG_INDX_NEWER_FILE,
+  IMG_INDX_NEWER_FOLDER,
+  IMG_INDX_MODIFIED_NEWER,
+  IMG_INDX_EXTERNALS_FOLDER,
+  IMG_INDX_COUNT
 };
 
+
 /**
- * array with column names. These names are
- * used to save and retrieve column specific
- * preferences.
- *
- * Note: these string must not be translatable
- *       so dont enclose them in _( )
+ * structure that maps a status entry to an 
+ * according XPM icon.
  */
-static const wxChar *
-COLUMN_NAMES[FileListCtrl::COL_COUNT] =
+struct MapItem
 {
-  wxT("Name"),
-  wxT("Path"),
-  wxT("Revision"),
-  wxT("RepRev"),
-  wxT("Author"),
-  wxT("Status"),
-  wxT("PropStatus"),
-  wxT("LastChanged"),
-  wxT("Extension"),
-  wxT("Date"),
-  wxT("PropDate"),
-  wxT("Checksum"),
-  wxT("Url"),
-  wxT("Repository"),
-  wxT("UUID"),
-  wxT("Schedule"),
-  wxT("Copied"),
-  wxT("ConflictOld"),
-  wxT("ConflictNew"),
-  wxT("ConflictWork")
+  int status;
+  const char ** xpm_data;
+
+  MapItem (int status_, const char **xpm_data_)
+    : status (status_), xpm_data (xpm_data_)
+  {
+  }
 };
 
 
+/** array of icons and corresponding status */
+static const MapItem MAP_ICON_ARRAY [] =
+{
+  MapItem (svn_wc_status_unversioned,           (const char**)nonsvn_file_xpm),
+  MapItem (svn_wc_status_normal,                (const char**)normal_file_xpm),
+  MapItem (svn_wc_status_added,                 (const char**)added_file_xpm),
+  MapItem (svn_wc_status_missing,               (const char**)missing_file_xpm),
+  MapItem (svn_wc_status_deleted,               (const char**)deleted_file_xpm),
+  MapItem (svn_wc_status_replaced,              (const char**)replaced_file_xpm),
+  MapItem (svn_wc_status_modified,              (const char**)modified_file_xpm),
+  MapItem (svn_wc_status_merged,                (const char**)merged_file_xpm),
+  MapItem (svn_wc_status_conflicted,            (const char**)conflicted_file_xpm),
+
+  MapItem (IMG_INDX_FOLDER,                     (const char**)folder_xpm),
+  MapItem (IMG_INDX_VERSIONED_FOLDER,           (const char**)versioned_folder_xpm),
+
+  MapItem (IMG_INDX_SORT_DOWN,                  (const char**)sort_down_xpm),
+  MapItem (IMG_INDX_SORT_UP,                    (const char**)sort_up_xpm),
+
+  MapItem (IMG_INDX_MODIFIED_VERSIONED_FOLDER,  (const char**)modified_versioned_folder_xpm),
+  MapItem (IMG_INDX_NEWER_FILE,                 (const char**)newer_file_xpm),
+  MapItem (IMG_INDX_NEWER_FOLDER,               (const char**)newer_folder_xpm),
+  MapItem (IMG_INDX_MODIFIED_NEWER,             (const char**)modified_newer_file_xpm),
+  MapItem (IMG_INDX_EXTERNALS_FOLDER,           (const char**)externals_folder_xpm)
+};
+
+static const size_t MAP_ICON_COUNT = 
+  sizeof (MAP_ICON_ARRAY) / sizeof (MAP_ICON_ARRAY [0]);
+
+/** the same for icons with local lock */
+static const MapItem MAP_LOCK_ICON_ARRAY [] =
+{
+  MapItem (svn_wc_status_normal,                (const char**)locked_normal_file_xpm),
+  MapItem (svn_wc_status_missing,               (const char**)locked_missing_file_xpm),
+  MapItem (svn_wc_status_deleted,               (const char**)locked_deleted_file_xpm),
+  MapItem (svn_wc_status_replaced,              (const char**)locked_replaced_file_xpm),
+  MapItem (svn_wc_status_modified,              (const char**)locked_modified_file_xpm),
+  MapItem (svn_wc_status_merged,                (const char**)locked_merged_file_xpm),
+  MapItem (svn_wc_status_conflicted,            (const char**)locked_conflicted_file_xpm),
+  MapItem (IMG_INDX_NEWER_FILE,                 (const char**)locked_newer_file_xpm),
+  MapItem (IMG_INDX_MODIFIED_NEWER,             (const char**)locked_modified_newer_file_xpm)
+};
+
+static const size_t MAP_LOCK_ICON_COUNT = 
+  sizeof (MAP_LOCK_ICON_ARRAY) / sizeof (MAP_LOCK_ICON_ARRAY [0]);
+
+/** the same for icons with repository lock */
+static const MapItem MAP_REPLOCK_ICON_ARRAY [] =
+{
+  MapItem (svn_wc_status_normal,                (const char**)replock_normal_file_xpm),
+  MapItem (svn_wc_status_missing,               (const char**)replock_missing_file_xpm),
+  MapItem (svn_wc_status_deleted,               (const char**)replock_deleted_file_xpm),
+  MapItem (svn_wc_status_replaced,              (const char**)replock_replaced_file_xpm),
+  MapItem (svn_wc_status_modified,              (const char**)replock_modified_file_xpm),
+  MapItem (svn_wc_status_merged,                (const char**)replock_merged_file_xpm),
+  MapItem (svn_wc_status_conflicted,            (const char**)replock_conflicted_file_xpm),
+  MapItem (IMG_INDX_NEWER_FILE,                 (const char**)replock_newer_file_xpm),
+  MapItem (IMG_INDX_MODIFIED_NEWER,             (const char**)replock_modified_newer_file_xpm)
+};
+
+static const size_t MAP_REPLOCK_ICON_COUNT = 
+  sizeof (MAP_REPLOCK_ICON_ARRAY) / sizeof (MAP_REPLOCK_ICON_ARRAY [0]);
+
 
 /**
  * private struct that hide implementation details
@@ -579,172 +618,371 @@ public:
   wxString Path;
   bool WithUpdate;
   bool ShowUnversioned;
+  bool IgnoreExternals;
   wxImageList *ImageListSmall;
-  bool ColumnVisible[COL_COUNT];
-  wxString ColumnCaption[COL_COUNT];
-  int ColumnIndex[COL_COUNT];
-  int ColumnWidth[COL_COUNT];
 
-  /** default constructor */
-  Data ()
-    : SortIncreasing (true), SortColumn (COL_NAME),
-      DirtyColumns (true), FlatMode (false), Context (0),
-      WithUpdate (false)
-  {
-    ImageListSmall = new wxImageList (16, 16, TRUE);
+  // no need to be static because FileListCtrl is created only once
+  const wxChar * COLUMN_CAPTIONS [FileListCtrl::COL_COUNT];
 
-    // add images to image list
-    ImageListSmall->Add (wxIcon (nonsvn_file_xpm));
-    ImageListSmall->Add (wxIcon (normal_file_xpm));
-    ImageListSmall->Add (wxIcon (added_file_xpm));
-    ImageListSmall->Add (wxIcon (missing_file_xpm));
-    ImageListSmall->Add (wxIcon (deleted_file_xpm));
-    ImageListSmall->Add (wxIcon (replaced_file_xpm));
-    ImageListSmall->Add (wxIcon (modified_file_xpm));
-    ImageListSmall->Add (wxIcon (merged_file_xpm));
-    ImageListSmall->Add (wxIcon (conflicted_file_xpm));
+  void InitColumnCaptions ();
 
-    ImageListSmall->Add (wxIcon (folder_xpm));
-    ImageListSmall->Add (wxIcon (versioned_folder_xpm));
-
-    ImageListSmall->Add (wxIcon (sort_down_xpm));
-    ImageListSmall->Add (wxIcon (sort_up_xpm));
-
-    ImageListSmall->Add (wxIcon (modified_versioned_folder_xpm));
-
-    ImageListSmall->Add (wxIcon (newer_file_xpm));
-    ImageListSmall->Add (wxIcon (newer_folder_xpm));
-    ImageListSmall->Add (wxIcon (modified_newer_file_xpm));
-  }
-
-  /** destructor */
-  ~Data ()
-  {
-    delete ImageListSmall;
-  }
+  static const wxChar * COLUMN_NAMES[FileListCtrl::COL_COUNT];
 
   /**
-   * get the real column. @a clickedColumn is only the index
-   * of the visible column that got clicked. If there are
-   * invisible columns this translates to something completely different
+   * This table holds information about image index in a image list.
+   * It will be accessed using a status code as a first index and locked
+   * flag as a second index.
+   * For every index that equals a valid status code there should be
+   * a valid value which represents an index in the image list
+   * when the second index is 0 (meaning that file is not locked).
+   * When second index equals 1 (for locked files), there are
+   * corresponding image indeces for some states.
    */
+  std::map<int,int> ImageIndexArray;
+
+  bool ColumnVisible [COL_COUNT];
+  wxString ColumnCaption [COL_COUNT];
+  int ColumnIndex [COL_COUNT];
+  int ColumnWidth [COL_COUNT];
+
+
+  Data ();
+  ~Data ();
+
   int
-  GetRealColumn (int clickedColumn)
-  {
-    int colIndex;
-    int column = clickedColumn;
+  GetImageIndex (const svn::Status & status);
 
-    for (colIndex = 0; colIndex <= clickedColumn; colIndex++)
-    {
-      if (!ColumnVisible[colIndex])
-      {
-        column++;
-      }
-    }
+  int
+  GetSortImageIndex (bool sortDown);
 
-    return column;
-  }
+  int
+  GetRealColumn (int clickedColumn);
 
-  /**
-   * callback function for @a wxListCtrl::SortColumns
-   */
   static int wxCALLBACK
-  CompareFunction (long item1, long item2, long sortData)
-  {
-    svn::Status * ps1 = (svn::Status *) item1;
-    svn::Status * ps2 = (svn::Status *) item2;
-    Data *data = (Data *) (sortData);
+  CompareFunction (long item1, long item2, long sortData);
 
-    if (ps1 && ps2)
-      return CompareItems (ps1, ps2, data->SortColumn,
-                           data->SortIncreasing);
-    else
-      return 0;
-  }
-
-  /**
-   * read the preferences for the filelist
-   * in this case the visibilty and width of the columns
-   */
   void
-  ReadConfig ()
-  {
-      // Get settings from config file:
-    wxConfigBase *config = wxConfigBase::Get ();
-    config->Read (ConfigWithUpdate,      &WithUpdate);
-    config->Read (ConfigFlatView,        &FlatMode);
-    config->Read (ConfigShowUnversioned, &ShowUnversioned, (bool) true);
+  ReadConfig ();
 
-    SortColumn = config->Read (ConfigSortColumn, (long) 0);
-    SortIncreasing = config->Read (ConfigSortOrder, (long) 1) ? true : false;
-
-    int col;
-    for (col = 0; col < COL_COUNT; col++)
-    {
-      wxString key;
-      key.Printf (ConfigColumnVisibleFmt, COLUMN_NAMES[col]);
-      ColumnVisible[col] = config->Read (key, (long) 1) != 0;
-
-      key.Printf (ConfigColumnWidthFmt, COLUMN_NAMES[col]);
-      long width = (long)GetDefaultWidth (col);
-      ColumnWidth[col] = config->Read (key, width);
-    }
-    ColumnVisible[COL_PATH] = FlatMode;
-  }
-
-  /**
-   * save the preferences for the filelist
-   */
   void
-  WriteConfig ()
-  {
-    // Write settings to config file:
-    wxConfigBase *config = wxConfigBase::Get ();
-    config->Write (ConfigWithUpdate,      WithUpdate);
-    config->Write (ConfigFlatView,        FlatMode);
-    config->Write (ConfigShowUnversioned, ShowUnversioned);
-
-    config->Write (ConfigSortColumn, (long) SortColumn);
-    config->Write (ConfigSortOrder, (long) (SortIncreasing ? 1 : 0));
-    // loop through all the columns
-    for (int col=0; col < COL_COUNT; col++)
-    {
-      wxString key;
-      key.Printf (ConfigColumnWidthFmt, COLUMN_NAMES[col]);
-      config->Write (key, (long) ColumnWidth[col]);
-
-      key.Printf (ConfigColumnVisibleFmt, COLUMN_NAMES[col]);
-      config->Write (key, (long) ColumnVisible[col]);
-    }
-  }
+  WriteConfig ();
 };
 
 
+/** default constructor */
+FileListCtrl::Data::Data ()
+: SortIncreasing (true), SortColumn (COL_NAME),
+  DirtyColumns (true), FlatMode (false), Context (0),
+  WithUpdate (false), IgnoreExternals (false)
+{
+  ImageListSmall = new wxImageList (16, 16, TRUE);
+
+  /**
+   * form neccessary ImageIndexArray and ImageListSmall arrays
+   * from two previous arrays, first common, than for locked items.
+   */
+  size_t i;
+  size_t lock_offset = 0;
+  for(i=0; i < MAP_ICON_COUNT; i++)
+  {
+    const MapItem & item = MAP_ICON_ARRAY [i];
+
+    ImageIndexArray [item.status + lock_offset] = i;
+    ImageListSmall->Add (wxIcon (item.xpm_data));
+  }
+
+  lock_offset += IMG_INDX_COUNT;
+  for(i=0; i < MAP_LOCK_ICON_COUNT; i++)
+  {
+    const MapItem & item = MAP_LOCK_ICON_ARRAY [i];
+
+    ImageIndexArray [item.status + lock_offset] = MAP_ICON_COUNT + i;
+    ImageListSmall->Add (wxIcon (item.xpm_data));
+  }
+
+  lock_offset += IMG_INDX_COUNT;
+  for(i=0; i < MAP_REPLOCK_ICON_COUNT; i++)
+  {
+    const MapItem & item = MAP_REPLOCK_ICON_ARRAY [i];
+
+    ImageIndexArray [item.status + lock_offset] = MAP_LOCK_ICON_COUNT + MAP_ICON_COUNT + i;
+    ImageListSmall->Add (wxIcon (item.xpm_data));
+  }
+
+  InitColumnCaptions ();
+}
+
+/** destructor */
+FileListCtrl::Data::~Data ()
+{
+  delete ImageListSmall;
+}
+
+void
+FileListCtrl::Data::InitColumnCaptions ()
+{
+  const wxChar *
+  COLUMN_CAPTIONS_INIT [] =
+  {
+    _("Name"),
+    _("Path"),
+    _("Revision"),
+    _("Rep. Rev."),
+    _("Author"),
+    _("Status"),
+    _("Prop Status"),
+    _("Last Changed"),
+    _("Extension"),
+    _("Date"),
+    _("Prop Date"),
+    _("Lock Owner"),
+    _("Lock Comment"),
+    _("Checksum"),
+    _("URL"),
+    _("Repository"),
+    _("UUID"),
+    _("Schedule"),
+    _("Copied"),
+    _("Conflict BASE"),
+    _("Conflict HEAD"),
+    _("Conflict Work")
+  };
+
+  // initializing FileListCtrl::COLUMN_CAPTIONS
+  for (size_t i = 0; i < FileListCtrl::COL_COUNT; i++)
+    COLUMN_CAPTIONS [i] = COLUMN_CAPTIONS_INIT [i];
+}
+
+/**
+ * array with column names. These names are
+ * used to save and retrieve column specific
+ * preferences.
+ *
+ * Note: these string must not be translatable
+ *       so dont enclose them in _( )
+ */
+const wxChar *
+FileListCtrl::Data::COLUMN_NAMES[FileListCtrl::COL_COUNT] =
+{
+  wxT("Name"),
+  wxT("Path"),
+  wxT("Revision"),
+  wxT("RepRev"),
+  wxT("Author"),
+  wxT("Status"),
+  wxT("PropStatus"),
+  wxT("LastChanged"),
+  wxT("Extension"),
+  wxT("Date"),
+  wxT("PropDate"),
+  wxT("LockOwner"),
+  wxT("LockComment"),
+  wxT("Checksum"),
+  wxT("URL"),
+  wxT("Repository"),
+  wxT("UUID"),
+  wxT("Schedule"),
+  wxT("Copied"),
+  wxT("ConflictOld"),
+  wxT("ConflictNew"),
+  wxT("ConflictWork")
+};
+
 /**
  * A safe wrapper for getting images - avoids array bounds
  * exceptions.
  */
-static int
-GetImageIndex (int textIndex, int propIndex)
+int
+FileListCtrl::Data::GetImageIndex (const svn::Status & status)
 {
-  int image = 0;
-  if ((textIndex >= 0) && (textIndex <= IMG_INDX_COUNT))
+  int imageIndex = 0;
+
+  size_t lock_offset = 0;
+  if (status.isRepLock ())
+    lock_offset = 2 * IMG_INDX_COUNT;
+  else if (status.isLocked ())
+    lock_offset = IMG_INDX_COUNT;
+
+  bool newer =
+    (status.reposTextStatus () == svn_wc_status_modified) ||
+    (status.reposPropStatus () == svn_wc_status_modified);
+
+  if (status.isVersioned ())
   {
-    if ( (textIndex == svn_wc_status_normal) &&
-         (propIndex > svn_wc_status_normal) )
+    int textIndex = status.textStatus ();
+    int propIndex = status.propStatus ();
+
+    if (IsDir (&status))
     {
-      if ( (propIndex >= 0) && (propIndex <= IMG_INDX_COUNT) )
+      if ((textIndex == svn_wc_status_modified) || (propIndex == svn_wc_status_modified))
       {
-        image = IMAGE_INDEX[propIndex];
+        imageIndex = ImageIndexArray[IMG_INDX_MODIFIED_VERSIONED_FOLDER + lock_offset];
       }
+      else if (textIndex == svn_wc_status_external)
+      {
+        imageIndex = ImageIndexArray[IMG_INDX_EXTERNALS_FOLDER + lock_offset];
+      }
+      else
+      {
+        imageIndex = ImageIndexArray[IMG_INDX_VERSIONED_FOLDER + lock_offset];
+      }
+      const int normalIndex = ImageIndexArray[IMG_INDX_VERSIONED_FOLDER + lock_offset];
+
+      if ((imageIndex == normalIndex) && newer)
+        imageIndex = ImageIndexArray[IMG_INDX_NEWER_FOLDER + lock_offset];
     }
     else
     {
-      image = IMAGE_INDEX[textIndex];
+      if ((textIndex >= 0) && (textIndex <= IMG_INDX_COUNT))
+      {
+        if ((textIndex == svn_wc_status_normal) && (propIndex > svn_wc_status_normal))
+        {
+          if ((propIndex >= 0) && (propIndex <= IMG_INDX_COUNT) )
+            imageIndex = ImageIndexArray[propIndex + lock_offset];
+        }
+        else
+          imageIndex = ImageIndexArray[textIndex + lock_offset];
+      }
+   
+      if (newer)
+      {
+        if (imageIndex == ImageIndexArray[svn_wc_status_normal + lock_offset])
+          imageIndex = ImageIndexArray[IMG_INDX_NEWER_FILE + lock_offset];
+        else
+          imageIndex = ImageIndexArray[IMG_INDX_MODIFIED_NEWER + lock_offset];
+      }
+    }
+  }
+  else
+  {
+    wxString wxFullPath = Utf8ToLocal (status.path ());
+
+    // unversioned entries dont carry dir info
+    // with them. must find this out by ourself
+    if (wxDirExists (wxFullPath))
+    {
+      imageIndex = ImageIndexArray[IMG_INDX_FOLDER + lock_offset];
+    }
+    else if (wxFileExists (wxFullPath))
+    {
+      imageIndex = ImageIndexArray[svn_wc_status_unversioned + lock_offset];
+    }
+    else
+    {
+      imageIndex = ImageIndexArray[svn_wc_status_none + lock_offset];
     }
   }
 
-  return image;
+  return imageIndex;
+}
+
+/**
+ * A safe wrapper for getting images for sorting.
+ */
+inline int
+FileListCtrl::Data::GetSortImageIndex (bool sortDown)
+{
+  if (sortDown)
+    return ImageIndexArray[IMG_INDX_SORT_DOWN];
+
+  return ImageIndexArray[IMG_INDX_SORT_UP];
+}
+
+/**
+ * get the real column. @a clickedColumn is only the index
+ * of the visible column that got clicked. If there are
+ * invisible columns this translates to something completely different
+ */
+inline int
+FileListCtrl::Data::GetRealColumn (int clickedColumn)
+{
+  int colIndex;
+  int column = clickedColumn;
+
+  for (colIndex = 0; colIndex <= clickedColumn; colIndex++)
+  {
+    if (!ColumnVisible[colIndex])
+    {
+      column++;
+    }
+  }
+
+  return column;
+}
+
+/**
+ * callback function for @a wxListCtrl::SortColumns
+ */
+int wxCALLBACK
+FileListCtrl::Data::CompareFunction (long item1, long item2, long sortData)
+{
+  svn::Status * ps1 = (svn::Status *) item1;
+  svn::Status * ps2 = (svn::Status *) item2;
+  Data *data = (Data *) (sortData);
+
+  if (ps1 && ps2)
+    return CompareItems (ps1, ps2, data->SortColumn,
+                         data->SortIncreasing);
+  else
+    return 0;
+}
+
+/**
+ * read the preferences for the filelist
+ * in this case the visibilty and width of the columns
+ */
+inline void
+FileListCtrl::Data::ReadConfig ()
+{
+  // Get settings from config file:
+  wxConfigBase *config = wxConfigBase::Get ();
+  config->Read (ConfigWithUpdate,      &WithUpdate);
+  config->Read (ConfigFlatView,        &FlatMode);
+  config->Read (ConfigShowUnversioned, &ShowUnversioned, (bool) true); 
+  config->Read (ConfigIgnoreExternals, &IgnoreExternals, (bool) false);
+
+  SortColumn = config->Read (ConfigSortColumn, (long) 0);
+  SortIncreasing = config->Read (ConfigSortOrder, (long) 1) ? true : false;
+
+  int col;
+  for (col = 0; col < COL_COUNT; col++)
+  {
+    wxString key;
+    key.Printf (ConfigColumnVisibleFmt, COLUMN_NAMES[col]);
+    ColumnVisible[col] = config->Read (key, (long) 1) != 0;
+
+    key.Printf (ConfigColumnWidthFmt, COLUMN_NAMES[col]);
+    long width = (long)GetDefaultWidth (col);
+    ColumnWidth[col] = config->Read (key, width);
+  }
+  ColumnVisible[COL_PATH] = FlatMode;
+}
+
+/**
+ * save the preferences for the filelist
+ */
+inline void
+FileListCtrl::Data::WriteConfig ()
+{
+  // Write settings to config file:
+  wxConfigBase *config = wxConfigBase::Get ();
+  config->Write (ConfigWithUpdate,      WithUpdate);
+  config->Write (ConfigFlatView,        FlatMode);
+  config->Write (ConfigShowUnversioned, ShowUnversioned);
+  config->Write (ConfigIgnoreExternals, IgnoreExternals);
+
+  config->Write (ConfigSortColumn, (long) SortColumn);
+  config->Write (ConfigSortOrder, (long) (SortIncreasing ? 1 : 0));
+  // loop through all the columns
+  for (int col=0; col < COL_COUNT; col++)
+  {
+    wxString key;
+    key.Printf (ConfigColumnWidthFmt, COLUMN_NAMES[col]);
+    config->Write (key, (long) ColumnWidth[col]);
+
+    key.Printf (ConfigColumnVisibleFmt, COLUMN_NAMES[col]);
+    config->Write (key, (long) ColumnVisible[col]);
+  }
 }
 
 
@@ -763,29 +1001,6 @@ FileListCtrl::FileListCtrl (wxWindow * parent, const wxWindowID id,
 {
   m = new Data ();
 
-  // set the indexes
-  IMAGE_INDEX[svn_wc_status_none] = 0;
-  IMAGE_INDEX[svn_wc_status_unversioned] = 0;
-  IMAGE_INDEX[svn_wc_status_normal] = 1;
-  IMAGE_INDEX[svn_wc_status_added] = 2;
-  IMAGE_INDEX[svn_wc_status_missing] = 3;
-  IMAGE_INDEX[svn_wc_status_deleted] = 4;
-  IMAGE_INDEX[svn_wc_status_replaced] = 5;
-  IMAGE_INDEX[svn_wc_status_modified] = 6;
-  IMAGE_INDEX[svn_wc_status_merged] = 7;
-  IMAGE_INDEX[svn_wc_status_conflicted] = 8;
-
-  IMAGE_INDEX[IMG_INDX_FOLDER] = 9;
-  IMAGE_INDEX[IMG_INDX_VERSIONED_FOLDER] = 10;
-
-  IMAGE_INDEX[IMG_INDX_SORT_DOWN] = 11;
-  IMAGE_INDEX[IMG_INDX_SORT_UP] = 12;
-
-  IMAGE_INDEX[IMG_INDX_MODIFIED_VERSIONED_FOLDER] = 13;
-  IMAGE_INDEX[IMG_INDX_NEWER_FILE] = 14;
-  IMAGE_INDEX[IMG_INDX_NEWER_FOLDER] = 15;
-  IMAGE_INDEX[IMG_INDX_MODIFIED_NEWER] = 16;
-
   // set this file list control to use the image list
   SetImageList (m->ImageListSmall, wxIMAGE_LIST_SMALL);
 
@@ -813,9 +1028,8 @@ FileListCtrl::UpdateFileList (const wxString & path)
 void
 FileListCtrl::UpdateFileList ()
 {
-  std::string pathUtf8;
-  LocalToUtf8 (m->Path, pathUtf8);
-  const bool isNative (!svn::Url::isValid (pathUtf8.c_str ()));
+  svn::Path pathUtf8 (PathUtf8 (m->Path));
+  const bool isNative (!pathUtf8.isUrl ());
   // delete all the items in the list to display the new ones
   DeleteAllItems ();
 
@@ -828,10 +1042,9 @@ FileListCtrl::UpdateFileList ()
 
   svn::Client client (m->Context);
   const svn::StatusEntries statusVector =
-    client.status (pathUtf8.c_str (), m->FlatMode, true, m->WithUpdate);
+    client.status (pathUtf8.c_str (), m->FlatMode, true, m->WithUpdate, false, m->IgnoreExternals);
   svn::StatusEntries::const_iterator it;
   const size_t pathUtf8Length = pathUtf8.length () + 1;
-
   for (it = statusVector.begin (); it != statusVector.end (); it++)
   {
     const svn::Status & status = *it;
@@ -855,7 +1068,6 @@ FileListCtrl::UpdateFileList ()
       values[COL_NAME] = wxFileNameFromPath (wxstr);
     }
 
-    wxString wxFullPath = Utf8ToLocal (status.path ());
     if (m->ColumnVisible[COL_PATH] || m->ColumnVisible[COL_EXTENSION])
     {
       svn::Path path (fullPath.c_str ());
@@ -863,72 +1075,24 @@ FileListCtrl::UpdateFileList ()
       path.split (dir, filename, ext);
 
       if (isNative)
-        values[COL_PATH]    = Utf8ToLocal (path.native ().c_str ());
+      { 
+        if (path.native ().length () > pathUtf8Length)
+          values[COL_PATH]  = Utf8ToLocal (path.native ().substr (pathUtf8Length).c_str ());
+        else
+          values[COL_PATH]  = Utf8ToLocal (".");
+      }
       else
-        values[COL_PATH]    = wxFullPath;
+        values[COL_PATH]    = wxString (Utf8ToLocal (fullPath.substr (pathUtf8Length)));
       values[COL_EXTENSION] = Utf8ToLocal (ext.c_str ());
     }
 
-    int imageIndex = 0;
-    bool newer =
-      (status.reposTextStatus () == svn_wc_status_modified) ||
-      (status.reposPropStatus () == svn_wc_status_modified);
-
-    if (status.isVersioned ())
-    {
-      if (IsDir (&status))
-      {
-        if ((status.textStatus () == svn_wc_status_modified) ||
-            (status.propStatus () == svn_wc_status_modified))
-          imageIndex = GetImageIndex (
-            IMG_INDX_MODIFIED_VERSIONED_FOLDER, -1);
-        else
-          imageIndex = GetImageIndex (
-            IMG_INDX_VERSIONED_FOLDER, -1);
-
-        const int normalIndex = IMAGE_INDEX[IMG_INDX_VERSIONED_FOLDER];
-        if ((imageIndex == normalIndex) && newer)
-          imageIndex = IMG_INDX_NEWER_FOLDER;
-      }
-      else
-      {
-        imageIndex = GetImageIndex (
-          status.textStatus (), status.propStatus ());
-
-        if (newer)
-        {
-          if (imageIndex == IMAGE_INDEX[svn_wc_status_normal])
-            imageIndex = IMAGE_INDEX[IMG_INDX_NEWER_FILE];
-          else
-            imageIndex = IMAGE_INDEX[IMG_INDX_MODIFIED_NEWER];
-        }
-      }
-    }
-    else
-    {
-      // unversioned entries dont carry dir info
-      // with them. must find this out by ourself
-      if (wxDirExists (wxFullPath))
-      {
-        imageIndex = GetImageIndex (IMG_INDX_FOLDER, -1);
-      }
-      else if (wxFileExists (wxFullPath))
-      {
-        imageIndex = GetImageIndex (svn_wc_status_unversioned, -1);
-      }
-      else
-      {
-        imageIndex = svn_wc_status_none;
-      }
-    }
-
+    int imageIndex = m->GetImageIndex (status);
 
     // User want to see unversioned entries?
     if (status.isVersioned() || m->ShowUnversioned)
     {
       InsertItem (i, values[COL_NAME], imageIndex);
 
-
       // The item data will be used to sort the list:
       SetItemData (i, (long)new svn::Status (status));    // The control now owns this data
       // and must delete it in due course.
@@ -936,6 +1100,7 @@ FileListCtrl::UpdateFileList ()
       if (status.isVersioned ())
       {
         const svn::Entry & entry = status.entry ();
+
         values[COL_REV].Printf (wxT("%ld"), entry.revision ());
         values[COL_CMT_REV].Printf (wxT("%ld"), entry.cmtRev ());
 
@@ -946,6 +1111,12 @@ FileListCtrl::UpdateFileList ()
         values[COL_TEXT_TIME] = FormatDateTime (entry.textTime ());
         values[COL_PROP_TIME] = FormatDateTime (entry.propTime ());
 
+        if (status.isLocked ())
+        {
+          values[COL_LOCK_OWNER] = Utf8ToLocal (status.lockOwner ());
+          values[COL_LOCK_COMMENT] = Utf8ToLocal (status.lockComment ());
+        }
+
         values[COL_URL] = Utf8ToLocal (entry.url ());
         values[COL_REPOS] = Utf8ToLocal (entry.repos ());
         values[COL_UUID] = Utf8ToLocal (entry.uuid ());
@@ -1041,7 +1212,6 @@ FileListCtrl::OnDoubleClick (wxMouseEvent & event)
       event.Skip ();
 }
 
-
 void
 FileListCtrl::OnKeyDown (wxKeyEvent & event)
 {
@@ -1288,6 +1458,8 @@ FileListCtrl::ResetColumns ()
     case COL_CONFLICT_OLD:
     case COL_CONFLICT_NEW:
     case COL_CONFLICT_WRK:
+    case COL_LOCK_OWNER:
+    case COL_LOCK_COMMENT:
     case COL_EXTENSION:
       visible = false;
       break;
@@ -1320,8 +1492,8 @@ FileListCtrl::SetColumnImages ()
     item.m_mask = wxLIST_MASK_IMAGE;
     if (col == m->SortColumn)
     {
-      item.m_image = m->SortIncreasing ?
-        IMAGE_INDEX[IMG_INDX_SORT_DOWN] : IMAGE_INDEX[IMG_INDX_SORT_UP];
+      bool sortDown = m->SortIncreasing ? true : false;
+      item.m_image = m->GetSortImageIndex (sortDown);
     }
     else
     {
@@ -1409,7 +1581,7 @@ FileListCtrl::UpdateColumns ()
     {
       wxListItem item;
       item.m_mask = wxLIST_MASK_TEXT | wxLIST_MASK_WIDTH;
-      item.m_text = COLUMN_CAPTIONS[col];
+      item.m_text = m->COLUMN_CAPTIONS[col];
       item.m_width = m->ColumnWidth[col];
       SetColumn (index, item);
     }
@@ -1471,48 +1643,53 @@ FileListCtrl::IsFlat ()
   return m->FlatMode;
 }
 
-
 void
 FileListCtrl::SetContext (svn::Context * Context)
 {
   m->Context = Context;
 }
 
-
 svn::Context *
 FileListCtrl::GetContext () const
 {
   return m->Context;
 }
 
-
 void
 FileListCtrl::SetWithUpdate (bool value)
 {
   m->WithUpdate = value;
 }
 
-
 bool
 FileListCtrl::GetWithUpdate () const
 {
   return m->WithUpdate;
 }
 
-
 void
 FileListCtrl::SetShowUnversioned (bool value)
 {
   m->ShowUnversioned = value;
 }
 
-
 bool
 FileListCtrl::GetShowUnversioned () const
 {
   return m->ShowUnversioned;
 }
 
+void
+FileListCtrl::SetIgnoreExternals (bool value)
+{
+  m->IgnoreExternals = value;
+}
+
+bool
+FileListCtrl::GetIgnoreExternals () const
+{
+  return m->IgnoreExternals;
+}
 
 /* -----------------------------------------------------------------
  * local variables:
