@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -63,6 +75,8 @@ public:
     COL_EXTENSION,
     COL_TEXT_TIME,
     COL_PROP_TIME,
+    COL_LOCK_OWNER,
+    COL_LOCK_COMMENT,
     COL_CHECKSUM,
     COL_URL,
     COL_REPOS,
@@ -122,7 +136,6 @@ public:
    */
   void SetFlat (const bool flat);
 
-
   /**
    * shall we contact the repository when updating,
    * so we get information about out-dated-ness?
@@ -130,7 +143,6 @@ public:
   void
   SetWithUpdate (bool value);
 
-
   bool
   GetWithUpdate () const;
 
@@ -141,7 +153,13 @@ public:
   SetShowUnversioned (bool value);
 
   bool
-  GetShowUnversioned() const;
+  GetShowUnversioned () const;
+
+  void
+  SetIgnoreExternals (bool value);
+
+  bool
+  GetIgnoreExternals () const;
 
   /**
    * get the "flat" setting
