@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -11,6 +23,15 @@
  * ====================================================================
  */
 
+// wxWidgets
+#include "wx/wx.h"
+#include "wx/filename.h"
+#include "wx/dir.h"
+#include "wx/imaglist.h"
+#include "wx/treectrl.h"
+#include "wx/confbase.h"
+#include "wx/hashmap.h"
+
 // svncpp
 #include "svncpp/context.hpp"
 #include "svncpp/client.hpp"
@@ -19,19 +40,11 @@
 #include "svncpp/url.hpp"
 #include "svncpp/wc.hpp"
 
-// wxWidgets
-#include "wx/wx.h"
-#include "wx/filename.h"
-#include "wx/dir.h"
-#include "wx/imaglist.h"
-#include "wx/treectrl.h"
-
 // app
 #include "folder_browser.hpp"
 #include "folder_item_data.hpp"
 #include "ids.hpp"
 #include "utils.hpp"
-#include "bookmarks.hpp"
 #include "rapidsvn_app.hpp"
 #include "rapidsvn_frame.hpp"
 
@@ -45,6 +58,8 @@
 #include "res/bitmaps/modified_versioned_folder.xpm"
 #include "res/bitmaps/modified_open_folder.xpm"
 #include "res/bitmaps/bookmark.xpm"
+#include "res/bitmaps/externals_folder.xpm"
+#include "res/bitmaps/open_externals_folder.xpm"
 
 enum
 {
@@ -57,12 +72,22 @@ enum
   FOLDER_IMAGE_MODIFIED_FOLDER,
   FOLDER_IMAGE_MODIFIED_OPEN_FOLDER,
   FOLDER_IMAGE_BOOKMARK,
+  FOLDER_IMAGE_EXTERNALS_FOLDER,
+  FOLDER_IMAGE_OPEN_EXTERNALS_FOLDER,
   FOLDER_IMAGE_COUNT
 };
 
 static const unsigned int MAXLENGTH_BOOKMARK = 35;
 
 
+const static wxChar ConfigBookmarkFmt [] = wxT("/Bookmarks/Bookmark%ld");
+const static wxChar ConfigBookmarkCount [] = wxT("/Bookmarks/Count");
+const static wxChar ConfigFlatModeFmt [] = wxT("/Bookmarks/Bookmark%ldFlat");
+
+static const wxString EmptyString;
+
+
+// local functions
 static bool
 IsValidSeparator (const wxString & sep)
 {
@@ -70,19 +95,72 @@ IsValidSeparator (const wxString & sep)
 }
 
 
+/** 
+ * data structure that contains information about
+ * a single bookmark
+ */
+struct Bookmark
+{
+public:
+  svn::Context * context;
+  bool flatMode;
+
+
+  Bookmark (bool flatMode_=false)
+    : context (0), flatMode (flatMode_)
+  {
+  }
+
+
+  ~Bookmark ()
+  {
+    ClearContext ();
+  }
+
+
+  void
+  ClearContext ()
+  {
+    if (context != 0)
+    {
+      delete context;
+      context = 0;
+    }
+  }
+
+
+  void
+  SetAuthCache (bool value)
+  {
+    if (context != 0)
+      context->setAuthCache (value);
+  }
+
+};
+
+
+static Bookmark InvalidBookmark;
+
+
+WX_DECLARE_STRING_HASH_MAP (Bookmark, BookmarkHashMap);
+
 struct FolderBrowser::Data
 {
+private:
+  svn::Context * singleContext;
 public:
   wxWindow * window;
   svn::ContextListener * listener;
   wxTreeCtrl* treeCtrl;
   wxTreeItemId rootId;
   wxImageList* imageList;
-  Bookmarks bookmarks;
+  BookmarkHashMap bookmarks;
   svn::Context defaultContext;
+  bool useAuthCache;
+
 
   Data (wxWindow * window, const wxPoint & pos, const wxSize & size)
-    : window (window), listener (0)
+    : singleContext (0), window (window), listener (0), useAuthCache (true)
   {
     imageList = new wxImageList (16, 16, TRUE);
     imageList->Add (wxIcon (computer_xpm));
@@ -94,6 +172,8 @@ public:
     imageList->Add (wxIcon (modified_versioned_folder_xpm));
     imageList->Add (wxIcon (modified_open_folder_xpm));
     imageList->Add (wxIcon (bookmark_xpm));
+    imageList->Add (wxIcon (externals_folder_xpm));
+    imageList->Add (wxIcon (open_externals_folder_xpm));
 
     treeCtrl = new wxTreeCtrl (window, -1, pos, size,
                                wxTR_HAS_BUTTONS|wxTR_SINGLE);
@@ -110,6 +190,48 @@ public:
     DeleteAllItems ();
   }
 
+  /**
+   * add a new bookmark, but only if it doesnt
+   * exist yet. Add a new context as well.
+   *
+   * @param name full path/url of the bookmark
+   */
+  void
+  AddBookmark (wxString name, bool flatMode)
+  {
+    TrimString (name);
+
+    svn::Path nameUtf8 (PathUtf8 (name));
+    if (!nameUtf8.isUrl())
+    {
+      wxFileName filename (name);
+      name = filename.GetFullPath (wxPATH_NATIVE);
+    }
+
+    name = BeautifyPath (name);
+
+    bookmarks [name] = Bookmark (flatMode);
+    
+    if (singleContext == 0)
+      bookmarks [name].context = CreateContext ();
+  }
+
+  
+  /**
+   * factory method to create a new context
+   */
+  svn::Context *
+  CreateContext ()
+  {
+    svn::Context * context = new svn::Context ();
+
+    context->setAuthCache (useAuthCache);
+    context->setListener (listener);
+
+    return context;
+  }
+
+  
   const wxString
   GetPath ()
   {
@@ -141,56 +263,26 @@ public:
   }
 
   svn::Context *
-  GetContext ()
+  GetContext () 
   {
-    const FolderItemData * data = GetSelection ();
-    svn::Context * context = 0;
+    if (singleContext != 0)
+      return singleContext;
 
-    if (data == 0)
-    {
-      context = &defaultContext;
-    }
-    else
-    {
-      bool ok = true;
-      while (context == 0)
-      {
-        switch (data->getFolderType ())
-        {
-        case FOLDER_TYPE_BOOKMARKS:
-          context = &defaultContext;
-          break;
+    const wxString & path = GetSelectedBookmarkPath ();
 
-        case FOLDER_TYPE_NORMAL:
-          {
-            wxTreeItemId id = data->GetId ();
-            wxTreeItemId parentId = treeCtrl->GetItemParent (id);
-            data = GetItemData (parentId);
-          }
-          break;
+    if (path.Length () == 0)
+      return &defaultContext;
 
-        case FOLDER_TYPE_BOOKMARK:
-          context = bookmarks.GetContext (data->getPath ());
-          ok = context != 0;
+    BookmarkHashMap::iterator it = bookmarks.find (path);
 
-          break;
-        default:
-          ok = false;
-          break;
-        }
+    if (it == bookmarks.end ())
+      return 0;
 
-        if (!ok)
-          break;
-      }
-    }
-
-    if (context != 0)
-      context->setListener (listener);
-
-    return context;
+    return it->second.context;
   }
 
 
+
   void
   ShowMenu (wxPoint & pt)
   {
@@ -209,9 +301,12 @@ public:
     menu.AppendSeparator ();
     AppendMenuItem (menu, ID_AddRepoBookmark);
     AppendMenuItem (menu, ID_CreateRepository);
+    AppendMenuItem (menu, ID_Switch);
 
     if (type==FOLDER_TYPE_BOOKMARK)
     {
+      menu.AppendSeparator ();
+      AppendMenuItem (menu, ID_EditBookmark);
       AppendMenuItem (menu, ID_RemoveBookmark);
       menu.AppendSeparator ();
       AppendMenuItem (menu, ID_Login);
@@ -236,8 +331,8 @@ public:
       }
 
       wxMenuItem * item = new wxMenuItem (&menu, ID_Logout, label);
-      item->Enable (enabled);
       menu.Append (item);
+      item->Enable (enabled);
     }
 
     if ((type==FOLDER_TYPE_BOOKMARK)||(type==FOLDER_TYPE_NORMAL))
@@ -315,6 +410,11 @@ public:
     wxTreeItemId parentId = event.GetItem ();
     int type = FOLDER_TYPE_INVALID;
 
+    // If the parent is already expanded, 
+    // nothing has to be done here
+    if (treeCtrl->IsExpanded (parentId))
+      return;
+
     if(!rootId.IsOk ())
     {
       rootId = treeCtrl->GetRootItem ();
@@ -330,13 +430,12 @@ public:
     {
       case FOLDER_TYPE_BOOKMARKS:
       {
-        const int count = bookmarks.Count ();
-        int index;
+        BookmarkHashMap::iterator it = bookmarks.begin ();
 
-        for(index = 0; index < count; index++)
+        for (; it!= bookmarks.end (); it++)
         {
-          const wxString path
-              (bookmarks.GetBookmark (index));
+          const wxString & path = it->first;
+
           FolderItemData* data= new FolderItemData (FOLDER_TYPE_BOOKMARK,
                                                     path, path, TRUE);
           wxTreeItemId newId = treeCtrl->AppendItem (parentId, path,
@@ -355,10 +454,9 @@ public:
       try
       {
         const wxString& parentPath = parentData->getPath ();
-        std::string parentPathUtf8;
-        LocalToUtf8 (parentPath, parentPathUtf8);
+        svn::Path parentPathUtf8 (PathUtf8 (parentPath));
 
-        if ( svn::Url::isValid (parentPathUtf8.c_str ()))
+        if (parentPathUtf8.isUrl ())
           RefreshRepository (parentPath, parentId);
         else
           RefreshLocal (parentPath, parentId);
@@ -380,7 +478,7 @@ public:
   {
     wxTreeItemId parentId = event.GetItem ();
 
-    long cookie;
+    wxTreeItemIdValue cookie;
     wxTreeItemId id = treeCtrl->GetFirstChild (parentId, cookie);
 
     while(id.IsOk())
@@ -396,66 +494,71 @@ public:
   RefreshLocal (const wxString & parentPath,
                 const wxTreeItemId & parentId)
   {
-    svn::Client client (GetContext ());
-    wxDir dir (parentPath);
-    if(!dir.IsOpened ())
+    // If the parent is already expanded, 
+    // nothing has to be done here
+    if (treeCtrl->IsExpanded (parentId))
       return;
 
-    wxString filename;
-    bool ok = dir.GetFirst(&filename, wxEmptyString,
-                           wxDIR_DIRS);
-    bool parentHasSubdirectories = false;
+    svn::Client client (GetContext ());
+    svn::Path parentPathUtf8 (PathUtf8 (parentPath));
 
-    while(ok)
+    // Get status array for parent and all entries within it
+    svn::StatusEntries entries = 
+      client.status (parentPathUtf8.c_str (),
+                     false,      // Not recursive
+                     true,       // Get all entries
+                     false,      // Dont update from repository
+                     false);     // Use global ignores
+
+    svn::StatusEntries::iterator it;
+    for (it = entries.begin (); it != entries.end (); it++)
     {
-      if(filename != Utf8ToLocal (svn::Wc::ADM_DIR_NAME))
-      {
-        parentHasSubdirectories = true;
+      svn::Status& status = *it;
 
-        wxFileName path(parentPath, filename, wxPATH_NATIVE);
-        wxString fullPath = path.GetFullPath ();
-        std::string fullPathUtf8;
-        LocalToUtf8(fullPath, fullPathUtf8);
+      // Convert path from UTF8 to Local
+      wxFileName filename (Utf8ToFileName (status.path ()));
+      wxString path (filename.GetFullPath ());
+
+      // Only display versioned directories and exclude parent itself
+      if (((status.entry ().kind () == svn_node_dir) ||
+           (status.textStatus () == svn_wc_status_external)) && 
+          (parentPath != path))
+      {
         int image = FOLDER_IMAGE_FOLDER;
         int open_image = FOLDER_IMAGE_OPEN_FOLDER;
 
-        if (!svn::Wc::checkWc (fullPathUtf8.c_str ()))
+        if (status.textStatus () == svn_wc_status_external)
         {
-          image = FOLDER_IMAGE_NONSVN_FOLDER;
-          open_image = FOLDER_IMAGE_NONSVN_OPEN_FOLDER;
+          image = FOLDER_IMAGE_EXTERNALS_FOLDER;
+          open_image = FOLDER_IMAGE_OPEN_EXTERNALS_FOLDER;
         }
         else
         {
-          svn::Status status = client.singleStatus (fullPathUtf8.c_str ());
-          if ((status.textStatus () == svn_wc_status_modified) ||
-              (status.propStatus () == svn_wc_status_modified))
-          {
-            image = FOLDER_IMAGE_MODIFIED_FOLDER;
-            open_image = FOLDER_IMAGE_MODIFIED_OPEN_FOLDER;
-          }
+          image = FOLDER_IMAGE_FOLDER;
+          open_image = FOLDER_IMAGE_OPEN_FOLDER;
         }
 
-        FolderItemData * data =
-          new FolderItemData (FOLDER_TYPE_NORMAL,
-                              fullPath,
-                              filename, TRUE);
 
-        wxTreeItemId newId =
-          treeCtrl->AppendItem(
-            parentId, filename,
-            image, image, data);
+        if ((status.textStatus () == svn_wc_status_modified) ||
+            (status.propStatus () == svn_wc_status_modified))
+        {
+          image = FOLDER_IMAGE_MODIFIED_FOLDER;
+          open_image = FOLDER_IMAGE_MODIFIED_OPEN_FOLDER;
+        }
+
+        wxFileName filename (path);
+        FolderItemData * data = new FolderItemData (
+          FOLDER_TYPE_NORMAL, path, 
+          filename.GetFullName (), TRUE);
+
+        wxTreeItemId newId = treeCtrl->AppendItem(
+          parentId, filename.GetFullName (), image, image, data);
+
         treeCtrl->SetItemHasChildren (
-          newId, HasSubdirectories(fullPath) );
-        treeCtrl->SetItemImage (newId, open_image,
-                                wxTreeItemIcon_Expanded);
+          newId, HasSubdirectories (path));
+        treeCtrl->SetItemImage (newId, open_image, wxTreeItemIcon_Expanded);
       }
-
-      ok = dir.GetNext (&filename);
     }
-
-    // If no subdirectories, don't show the expander
-    if (!parentHasSubdirectories)
-      treeCtrl->SetItemHasChildren(parentId, FALSE);
   }
 
   void
@@ -464,8 +567,7 @@ public:
   {
     svn::Client client (GetContext ());
     svn::Revision rev (svn::Revision::HEAD);
-    std::string parentPathUtf8;
-    LocalToUtf8(parentPath, parentPathUtf8);
+    svn::Path parentPathUtf8 (PathUtf8 (parentPath));
     svn::DirEntries entries =
       client.ls (parentPathUtf8.c_str (), rev, false);
     svn::DirEntries::const_iterator it;
@@ -508,7 +610,7 @@ public:
   wxTreeItemId
   FindClosestChild (const wxTreeItemId & parentId, const wxString & path)
   {
-    long cookie;
+    wxTreeItemIdValue cookie;
     wxTreeItemId id = treeCtrl->GetFirstChild (parentId, cookie);
     wxTreeItemId childId;
 
@@ -558,15 +660,23 @@ public:
    * (folder removed or deleted) we open up the directory structure
    * as close as possible.
    *
+   * If @a path is an empty string, select the root
+   *
    * @param path
+   * @retval true selection successful
    */
   bool
   SelectFolder (const wxString & pathP)
   {
+    if (pathP.Length () == 0)
+    {
+      treeCtrl->SelectItem (treeCtrl->GetRootItem ());
+      return true;
+    }
+    
     // Convert the input string to subversion internal representation
-    std::string pathPUtf8;
-    LocalToUtf8 (pathP, pathPUtf8);
-    const wxString path = Utf8ToLocal (svn::Path (pathPUtf8).c_str());
+    svn::Path pathPUtf8 (PathUtf8 (pathP));
+    const wxString path = Utf8ToLocal (pathPUtf8.c_str());
 
     wxTreeItemId bookmarkId = GetSelectedBookmarkId ();
 
@@ -640,7 +750,7 @@ public:
   bool
   SelectBookmark (const wxString & bookmarkPath)
   {
-    long cookie;
+    wxTreeItemIdValue cookie;
     wxTreeItemId id = treeCtrl->GetFirstChild (rootId, cookie);
 
     bool success = false;
@@ -706,22 +816,98 @@ public:
   }
 
 
-  wxString
-  GetSelectedBookmark () const
+  const wxString & 
+  GetSelectedBookmarkPath () const
   {
     wxTreeItemId id = GetSelectedBookmarkId ();
 
-    wxString path;
+    if (!id.IsOk ())
+      return EmptyString;
 
-    if (id.IsOk ())
+    FolderItemData * data = GetItemData (id);
+
+    wxASSERT (data);
+
+    return data->getPath ();
+  }
+
+ 
+  Bookmark &
+  GetSelectedBookmark ()
+  {
+    const wxString & path = GetSelectedBookmarkPath ();
+
+    // empty string means there isnt a bookmark
+    // for the selecttion, e.g. the root of the tree
+    if (path.Length () == 0)
+      return InvalidBookmark;
+
+    // check whether we can find the bookmark
+    BookmarkHashMap::iterator it = bookmarks.find (path);
+
+    if (it == bookmarks.end ())
+      return InvalidBookmark;
+
+    return it->second;
+  }
+
+
+  void
+  SetAuthPerBookmark (const bool perBookmark)
+  {
+    if (!perBookmark)
     {
-      FolderItemData * data = GetItemData (id);
-
-      if (data != 0)
-        path = data->getPath ();
+      // one Context for all
+      ClearContexts ();
+      singleContext = CreateContext ();
     }
+    else
+    {
+      if (singleContext != 0)
+      {
+        delete singleContext;
+        singleContext = 0;
+      }
 
-    return path;
+      BookmarkHashMap::iterator it = bookmarks.begin ();
+
+      for (; it!=bookmarks.end (); it++)
+      {
+        it->second.context = CreateContext ();
+      }
+    }
+  }
+
+  const bool
+  GetAuthPerBookmark () const
+  {
+    return singleContext == 0;
+  }
+
+
+  void ClearContexts ()
+  {
+    BookmarkHashMap::iterator it = bookmarks.begin ();
+
+    for (; it!=bookmarks.end (); it++)
+      it->second.ClearContext ();
+  }
+
+
+  void SetAuthCache (bool value)
+  {
+    useAuthCache = value;
+
+    // make sure the already existing contexts
+    // are using the new setting
+    defaultContext.setAuthCache (value);
+    if (singleContext != 0)
+       singleContext->setAuthCache (value);
+
+    BookmarkHashMap::iterator it = bookmarks.begin ();
+
+    for (; it!= bookmarks.end (); it++)
+      it->second.SetAuthCache (value);
   }
 
 };
@@ -751,7 +937,7 @@ void
 FolderBrowser::Refresh ()
 {
   // remember selected
-  wxString bookmarkPath = m->GetSelectedBookmark ();
+  wxString bookmarkPath = m->GetSelectedBookmarkPath ();
   wxString path = m->GetPath ();
 
   // refresh contents
@@ -775,30 +961,29 @@ FolderBrowser::Refresh ()
 const bool
 FolderBrowser::RemoveBookmark ()
 {
-  bool success = FALSE;
-
   wxTreeItemId id = m->treeCtrl->GetSelection ();
 
-  if(id.IsOk ())
-  {
-    FolderItemData* data = (FolderItemData*) m->treeCtrl->GetItemData (id);
+  if(!id.IsOk ())
+    return false;
 
-    if( data->getFolderType() == FOLDER_TYPE_BOOKMARK )
-    {
-      wxString path = data->getPath ();
-      m->Delete (id);
-      m->bookmarks.RemoveBookmark (path);
-      success = TRUE;
-    }
-  }
+  FolderItemData* data = (FolderItemData*) m->treeCtrl->GetItemData (id);
 
-  return success;
+  if (data->getFolderType () != FOLDER_TYPE_BOOKMARK)
+    return false;
+
+  SelectFolder (wxEmptyString);
+
+  wxString path = data->getPath ();
+  m->Delete (id);
+  m->bookmarks.erase (path);
+
+  return true;
 }
 
 void
 FolderBrowser::AddBookmark (const wxString & path)
 {
-  m->bookmarks.AddBookmark (path);
+  m->AddBookmark (path, false);
 }
 
 const
@@ -872,18 +1057,6 @@ FolderBrowser::SelectFolder (const wxString & path)
   return m->SelectFolder (path);
 }
 
-const size_t
-FolderBrowser::GetBookmarkCount () const
-{
-  return m->bookmarks.Count ();
-}
-
-const wxString &
-FolderBrowser::GetBookmark (const size_t index) const
-{
-  return m->bookmarks.GetBookmark (index);
-}
-
 svn::Context *
 FolderBrowser::GetContext ()
 {
@@ -893,13 +1066,13 @@ FolderBrowser::GetContext ()
 void
 FolderBrowser::SetAuthPerBookmark (const bool value)
 {
-  m->bookmarks.SetAuthPerBookmark (value);
+  m->SetAuthPerBookmark (value);
 }
 
 const bool
 FolderBrowser::GetAuthPerBookmark () const
 {
-  return m->bookmarks.GetAuthPerBookmark ();
+  return m->GetAuthPerBookmark ();
 }
 
 bool
@@ -922,8 +1095,115 @@ FolderBrowser::GetListener () const
 }
 
 
+void
+FolderBrowser::WriteConfig (wxConfigBase * cfg) const
+{
+  wxASSERT (cfg);
+
+  cfg->Write (ConfigBookmarkCount, (long)m->bookmarks.size ());
+
+  // Save the bookmarks contents
+  long item = 0;
+
+  BookmarkHashMap::iterator it = m->bookmarks.begin ();
+
+  for (; it != m->bookmarks.end (); it++)
+  {
+    wxString keyPath, keyFlatMode;
+    keyPath.Printf (ConfigBookmarkFmt, item);
+    keyFlatMode.Printf (ConfigFlatModeFmt, item);
+
+    cfg->Write (keyPath, it->first);
+    if (it->second.flatMode)
+      cfg->Write (keyFlatMode, (long)1);
+    else
+      cfg->Write (keyFlatMode, (long)0);
+
+    item++;
+  }
+}
+
+
+void
+FolderBrowser::ReadConfig (wxConfigBase * cfg)
+{
+  wxASSERT (cfg);
+
+
+  long item, count;
+  cfg->Read (ConfigBookmarkCount, &count, 0);
+  for (item = 0; item < count; item++)
+  {
+    wxString keyPath, keyFlatMode;
+    wxString path;
+
+    keyPath.Printf (ConfigBookmarkFmt, item);
+    keyFlatMode.Printf (ConfigFlatModeFmt, item);
+    cfg->Read (keyPath, &path, wxEmptyString);
+
+    long flatMode;
+    cfg->Read (keyFlatMode, &flatMode, 0); 
+
+    if (path.Length () > 0)
+      m->bookmarks [path] = Bookmark (flatMode != 0);
+  }
+}
+
+
+const bool
+FolderBrowser::IsFlat () const
+{
+  Bookmark & bookmark = m->GetSelectedBookmark ();
+
+  if (&bookmark == &InvalidBookmark)
+    return false;
+
+  return bookmark.flatMode;
+}
+
+
+bool
+FolderBrowser::SetFlat (bool flatMode) 
+{
+  Bookmark & bookmark = m->GetSelectedBookmark ();
+
+  if (&bookmark == &InvalidBookmark)
+    return false;
+
+  bookmark.flatMode = flatMode;
+  return true;
+} 
+  
+
+const bool
+FolderBrowser::GetAuthCache () const
+{
+  return m->useAuthCache;
+}
+ 
+void
+FolderBrowser::SetAuthCache (const bool value)
+{
+  m->SetAuthCache (value);
+}
+
+void
+FolderBrowser::ExpandSelection ()
+{
+  if (!m->treeCtrl)
+    return;
+
+  wxTreeItemId id (m->treeCtrl->GetSelection ());
+
+  if (!id.IsOk ())
+    return;
+
+  m->treeCtrl->Expand (id);
+}
+
 /* -----------------------------------------------------------------
  * local variables:
  * eval: (load-file "../rapidsvn-dev.el")
  * end:
  */
+
