@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -19,6 +31,7 @@
 // forward declarations
 class wxImageList;
 class FolderItemData;
+class wxConfigBase;
 namespace svn
 {
   class Context;
@@ -35,39 +48,30 @@ public:
 
   virtual ~ FolderBrowser ();
 
-  virtual void Refresh ();
+  virtual void 
+  Refresh ();
 
   /**
    * remove the selected bookmark from bookmarks
    */
-  const bool RemoveBookmark ();
+  const bool 
+  RemoveBookmark ();
 
   /**
-   * add bookmark
+   * add bookmark. 
+   * 
+   * @param path path/url of the bookmark
    */
-  void AddBookmark (const wxString & path);
-
-  /**
-   * @return bookmark count on bookmarks
-   */
-  const size_t
-  GetBookmarkCount () const;
-
-  /**
-   * get the path of the bookmark at @a index
-   *
-   * @param index zero-based index
-   * @return path of the bookmark
-   */
-  const wxString &
-  GetBookmark (const size_t index) const;
+  void 
+  AddBookmark (const wxString & path);
 
   /**
    * returns the path of the current selection
    * if nothing is selected, an empty string
    * is returned
    */
-  const wxString GetPath () const;
+  const wxString 
+  GetPath () const;
 
   /**
    * returns the "context" of the current selection.
@@ -75,7 +79,8 @@ public:
    *
    * @return context of selection
    */
-  const FolderItemData * GetSelection () const;
+  const FolderItemData * 
+  GetSelection () const;
 
   /**
    * return the authentication context of the current selection
@@ -83,7 +88,8 @@ public:
    * @return authentication context
    * @retval NULL if Bookmarks is selected
    */
-  svn::Context * GetContext ();
+  svn::Context * 
+  GetContext ();
 
   /**
    * selects @a path in the current folder.
@@ -114,6 +120,26 @@ public:
   void
   SetAuthPerBookmark (const bool value);
 
+  /** 
+   * check whether flat-mode is turned on for the 
+   * selected bookmark
+   *
+   * @retval true flat-mode is turned on
+   */
+  const bool
+  IsFlat () const;
+
+  /**
+   * turns flat-mode for the selected bookmark 
+   * on or off
+   *
+   * @param flatMode true=on
+   * @retval true flat-mode successfully set
+   * @retval false not set, e.g. root selected or nothing
+   */
+  bool
+  SetFlat (bool flatMode);
+
   /**
    * @return auth per bookmark setting
    */
@@ -121,6 +147,18 @@ public:
   GetAuthPerBookmark () const;
 
 
+  /**
+   * check if authentication credentials should be cached or not
+   */
+  const bool
+  GetAuthCache () const;
+
+  /**
+   * select if authentication credentials should be cached or not
+   */
+  void
+  SetAuthCache (const bool value);
+
   void
   SetListener (svn::ContextListener * listener);
 
@@ -128,6 +166,34 @@ public:
   svn::ContextListener *
   GetListener () const;
 
+
+  /**
+   * Writes configuration including bookmarks
+   * to @a cfg
+   *
+   * @param cfg Config instance to write to
+   */
+  void
+  WriteConfig (wxConfigBase * cfg) const;
+
+
+  /**
+   * Read configuration
+   *
+   * @param cfg Config instance
+   */
+  void
+  ReadConfig (wxConfigBase * cfg);
+
+
+  /**
+   * Expand the selected leaf of the tree.
+   * Has no effect if the selection has no
+   * leafs
+   */
+  void
+  ExpandSelection ();
+
 private:
   struct Data;
   Data * m;
@@ -150,7 +216,7 @@ private:
 private:
   DECLARE_EVENT_TABLE ()
 
-  void OnSize (wxSizeEvent & WXUNUSED (event));
+  void OnSize (wxSizeEvent & event);
 
   void OnExpandItem (wxTreeEvent & event);
 
@@ -159,7 +225,6 @@ private:
   void OnItemRightClk (wxTreeEvent & event);
 
   void OnContextMenu (wxContextMenuEvent & event);
-
 };
 
 
