@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -27,6 +39,7 @@ enum
   ID_Refresh,
   ID_RefreshWithUpdate,
   ID_ShowUnversioned,
+  ID_IgnoreExternals,
   ID_HelpContents,
   ID_HelpIndex,
   ID_HelpStartupTips,
@@ -34,6 +47,7 @@ enum
   ID_AddWcBookmark,
   ID_AddRepoBookmark,
   ID_RemoveBookmark,
+  ID_EditBookmark,
   ID_Info,
   ID_Login,
   ID_Logout,
@@ -57,7 +71,9 @@ enum
   ID_DiffBase,
   ID_DiffHead,
   ID_Explore,
+  ID_Export,
   ID_Import,
+  ID_Lock,
   ID_Log,
   ID_Merge,
   ID_Mkdir,
@@ -70,6 +86,7 @@ enum
   ID_Revert,
   ID_Switch,
   ID_Update,
+  ID_Unlock,
   ID_Edit,
   ID_File_Max,
 
@@ -96,6 +113,8 @@ enum
   ID_Column_ConflictOld,
   ID_Column_ConflictNew,
   ID_Column_ConflictWrk,
+  ID_Column_LockOwner,
+  ID_Column_LockComment,
   ID_Column_Checksum,
   ID_Column_Max,
 
