@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -134,10 +146,12 @@ private:
     InsertColumn (0, _("Revision"));
     InsertColumn (1, _("User"));
     InsertColumn (2, _("Date"));
+    InsertColumn (3, _("Log Message"));
 
     SetColumnWidth (0, 65);
     SetColumnWidth (1, 100);
-    SetColumnWidth (2, 200);
+    SetColumnWidth (2, 150);
+    SetColumnWidth (3, 200);
 
     if (entries == 0)
       return;
@@ -158,6 +172,7 @@ private:
       InsertItem (index, rev);
       SetItem (index, 1, Utf8ToLocal (entry.author.c_str ()));
       SetItem (index, 2, dateStr);
+      SetItem (index, 3, Utf8ToLocal (entry.message.c_str ()));
       index++;
     }
  
@@ -216,7 +231,7 @@ public:
 
     wxBoxSizer * logSizer = new wxBoxSizer (wxVERTICAL);
     logSizer->Add (historyLabel, 0, wxALL, 5);
-    logSizer->Add (m_logList, 1, wxLEFT);
+    logSizer->Add (m_logList, 1, wxLEFT | wxEXPAND);
 
     wxBoxSizer * buttonSizer = new wxBoxSizer (wxVERTICAL);
     buttonSizer->Add (buttonClose,   0, wxALL | wxEXPAND, 5);
