@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -11,12 +23,12 @@
  * ====================================================================
  */
 
-// svncpp
-#include "svncpp/client.hpp"
-
 // wxWidgets
 #include "wx/wx.h"
 
+// svncpp
+#include "svncpp/client.hpp"
+
 // app
 #include "destination_dlg.hpp"
 #include "mkdir_action.hpp"
@@ -58,12 +70,11 @@ MkdirAction::Perform ()
   wxString newDir (m_path + m_target);
   // TODO: What is newDir for???
 
-  std::string pathUtf8 (LocalToUtf8 (m_path));
-  std::string targetUtf8 (LocalToUtf8 (m_target));
-  svn::Path target (pathUtf8);
-  target.addComponent (targetUtf8);
+  svn::Path pathUtf8 (PathUtf8 (m_path));
+  svn::Path targetUtf8 (PathUtf8 (m_target));
+  pathUtf8.addComponent (targetUtf8.c_str ());
 
-  client.mkdir (target, "");
+  client.mkdir (pathUtf8, "");
   return true;
 }
 /* -----------------------------------------------------------------
