@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -337,6 +349,15 @@ public:
                         wxDefaultSize, 0, valCheck);
       mainsizer->Add (20, 20);
       mainsizer->Add (check, 0, wxALL, 5);
+
+      wxGenericValidator cacheCheck (&m_prefs->useAuthCache);
+      check =
+        new wxCheckBox (this, -1,
+                        _("Store authentication credentials on hard disk"),
+                        wxDefaultPosition,
+                        wxDefaultSize, 0, cacheCheck);
+
+      mainsizer->Add (check, 0, wxALL, 5);
     }
 
     SetSizer (mainsizer);
@@ -375,7 +396,9 @@ public:
     wxBoxSizer *button_sizer = new wxBoxSizer (wxHORIZONTAL);
 
     notebook = new wxNotebook (wnd, -1, wxDefaultPosition, wxDefaultSize);
+#if !wxCHECK_VERSION(2, 6, 0)
     wxNotebookSizer *nbs = new wxNotebookSizer (notebook);
+#endif
 
     // Add the pages
     GeneralPanel *generalPanel = new GeneralPanel (notebook, prefs);
@@ -400,7 +423,11 @@ public:
     //ExternalsPanel *externalsPanel = ExternalsPanel::Create (notebook);
     //notebook->AddPage (externalsPanel, _("Externals"));
 
+#if wxCHECK_VERSION(2, 6, 0)
+    topsizer->Add (notebook, 1, wxALIGN_CENTER | wxEXPAND | wxALL, 6);
+#else
     topsizer->Add (nbs, 1, wxALIGN_CENTER | wxEXPAND | wxALL, 6);
+#endif
     topsizer->Add (button_sizer, 0, wxALIGN_CENTER);
 
     // Order is important here:
@@ -419,7 +446,12 @@ public:
     // Transfer data for every page. This doesn't happen by default.
     if (notebook)
     {
-      for (int i = 0; i < notebook->GetPageCount (); i++)
+#if wxCHECK_VERSION(2, 6, 0)
+      size_t i;
+#else
+      int i;
+#endif
+      for (i = 0; i < notebook->GetPageCount (); i++)
         b = b && notebook->GetPage (i)->TransferDataFromWindow ();
     }
     return b;
@@ -432,7 +464,12 @@ public:
     // Transfer data for every page. This doesn't happen by default.
     if (notebook)
     {
-      for (int i = 0; i < notebook->GetPageCount (); i++)
+#if wxCHECK_VERSION(2, 6, 0)
+      size_t i;
+#else
+      int i;
+#endif
+      for (i = 0; i < notebook->GetPageCount (); i++)
         b = b && notebook->GetPage (i)->TransferDataToWindow ();
     }
     return b;
