@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -36,7 +48,6 @@ public:
     : window (wnd), property (context, target)
   {
   }
-
 };
 
 PropertyDlg::PropertyDlg (wxWindow * parent,
@@ -73,8 +84,8 @@ PropertyDlg::ReadFromGrid ()
 {
   try
   {
-    const std::vector<svn::PropertyEntry> & entries
-      = m->property.entries ();
+    const std::vector<svn::PropertyEntry> & entries =
+      m->property.entries ();
     std::vector<svn::PropertyEntry>::const_iterator it = entries.begin ();
 
     // first check for deleted properties
@@ -112,7 +123,6 @@ PropertyDlg::ReadFromGrid ()
   {
     wxMessageBox (_("Error setting the property values"), _("Error"));
   }
-
 }
 
 /**
@@ -136,13 +146,12 @@ PropertyDlg::WriteToGrid ()
     const svn::PropertyEntry & entry = *it;
 
     SetEntry (Utf8ToLocal (entry.name.c_str ()),
-        Utf8ToLocal (entry.value.c_str ()));
+      Utf8ToLocal (entry.value.c_str ()));
 
     it++;
   }
 }
 
-
 /* -----------------------------------------------------------------
  * local variables:
  * eval: (load-file "../rapidsvn-dev.el")
