@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -19,6 +31,7 @@
 // svncpp
 #include "svncpp/apr.hpp"
 #include "svncpp/context.hpp"
+#include "svncpp/check.hpp"
 #include "svncpp/exception.hpp"
 #include "svncpp/targets.hpp"
 #include "svncpp/url.hpp"
@@ -27,10 +40,12 @@
 // app
 #include "rapidsvn_app.hpp"
 #include "config.hpp"
+#include "destination_dlg.hpp"
 #include "diff_data.hpp"
 #include "ids.hpp"
 #include "file_info.hpp"
 #include "listener.hpp"
+#include "folder_item_data.hpp"
 
 #ifdef USE_SIMPLE_WORKER
 #include "simple_worker.hpp"
@@ -48,6 +63,8 @@
 #include "external_program_action.hpp"
 #include "get_action.hpp"
 #include "import_action.hpp"
+#include "export_action.hpp"
+#include "lock_action.hpp"
 #include "merge_action.hpp"
 #include "mkdir_action.hpp"
 #include "move_action.hpp"
@@ -55,6 +72,7 @@
 #include "resolve_action.hpp"
 #include "revert_action.hpp"
 #include "switch_action.hpp"
+#include "unlock_action.hpp"
 #include "update_action.hpp"
 #include "view_action.hpp"
 
@@ -72,7 +90,7 @@
 #include "proportional_splitter.hpp"
 
 // Bitmaps
-#include "res/bitmaps/svn.xpm"
+#include "res/bitmaps/rapidsvn_16x16.xpm"
 #include "res/bitmaps/refresh.xpm"
 #include "res/bitmaps/update.xpm"
 #include "res/bitmaps/commit.xpm"
@@ -95,8 +113,6 @@ const static wxChar ConfigHeight[] = wxT("/MainFrame/Height");
 const static wxChar ConfigMaximized[] = wxT("/MainFrame/Maximized");
 const static wxChar ConfigSplitterHoriz[] = wxT("/MainFrame/SplitterHoriz");
 const static wxChar ConfigSplitterVert[] = wxT("/MainFrame/SplitterVert");
-const static wxChar ConfigBookmarkFmt[] = wxT("/Bookmarks/Bookmark%ld");
-const static wxChar ConfigBookmarkCount[] = wxT("/Bookmarks/Count");
 
 const static wxChar TraceMisc[] = wxT("tracemisc");
 
@@ -128,6 +144,8 @@ static const int COLUMN_ID_MAP[FileListCtrl::COL_COUNT] =
   ID_Column_Extension,
   ID_Column_Text_Time,
   ID_Column_Prop_Time,
+  ID_Column_LockOwner,
+  ID_Column_LockComment,
   ID_Column_Checksum,
   ID_Column_Url,
   ID_Column_Repos,
@@ -139,36 +157,6 @@ static const int COLUMN_ID_MAP[FileListCtrl::COL_COUNT] =
   ID_Column_ConflictWrk
 };
 
-/**
- * Captions for the columns. We cant use the texts used for
- * the column headings in FileListCtrl since they dont have
- * shortcuts.
- */
-static const wxChar *
-COLUMN_CAPTIONS[FileListCtrl::COL_COUNT] =
-{
-  wxT(""), // Name is not used here
-  wxT(""), // Path is not used here
-  _("&Revision"),
-  _("R&ep. Rev."),
-  _("&Author"),
-  _("&Status"),
-  _("&Prop Status"),
-  _("&Last Changed"),
-  _("E&xtension"),
-  _("&Date"),
-  _("Pr&op Date"),
-  _("Checksum"),
-  _("Url"),
-  _("Repository"),
-  _("UUID"),
-  _("Schedule"),
-  _("Copied"),
-  _("Conflict Old"),
-  _("Conflict New"),
-  _("Conflict Work")
-};
-
 
 /**
  * Local helper function to create the action worker
@@ -265,6 +253,38 @@ public:
     AppendMenuItem (*MenuColumns, ID_Column_Reset);
     MenuColumns->AppendSeparator ();
 
+    /**
+     * Captions for the columns. We cant use the texts used for
+     * the column headings in FileListCtrl since they dont have
+     * shortcuts.
+     */
+    const wxChar *
+    COLUMN_CAPTIONS[FileListCtrl::COL_COUNT] =
+    {
+      wxT(""), // Name is not used here
+      wxT(""), // Path is not used here
+      _("&Revision"),
+      _("R&ep. Rev."),
+      _("&Author"),
+      _("&Status"),
+      _("&Prop Status"),
+      _("&Last Changed"),
+      _("E&xtension"),
+      _("&Date"),
+      _("Pr&op Date"),
+      _("Lock Owner"),
+      _("Lock Comment"),
+      _("Checksum"),
+      _("URL"),
+      _("Repository"),
+      _("UUID"),
+      _("Schedule"),
+      _("Copied"),
+      _("Conflict BASE"),
+      _("Conflict HEAD"),
+      _("Conflict Work")
+    };
+
     for (int col = 0; col < FileListCtrl::COL_COUNT; col++)
     {
       int id = COLUMN_ID_MAP[col];
@@ -287,6 +307,8 @@ public:
     menuView->AppendCheckItem (ID_Flat, _("Flat Mode"));
     menuView->AppendCheckItem (ID_RefreshWithUpdate, _("Refresh with Update"));
     menuView->AppendCheckItem (ID_ShowUnversioned, _("Show unversioned entries"));
+    if (svn::SUPPORTS_EXTERNALS)
+      menuView->AppendCheckItem (ID_IgnoreExternals, _("Ignore Externals"));
 
     // Preferences menu item goes to its own place on MacOSX,
     // so no separator is necessary.
@@ -298,13 +320,14 @@ public:
     // Repository menu
     wxMenu *menuRepos = new wxMenu;
     menuRepos->Append (ID_Import, _("&Import...\tCTRL-I"));
+    menuRepos->Append (ID_Export, _("&Export...\tCTRL-E"));
     menuRepos->Append (ID_Checkout, _("Check&out...\tCTRL-O"));
     menuRepos->Append (ID_CreateRepository, _("&Create..."));
 
     menuRepos->AppendSeparator ();
 
     menuRepos->Append (ID_Merge, _("Merge..."));
-    menuRepos->Append (ID_Switch, _("Switch...\tCTRL-S"));
+    menuRepos->Append (ID_Switch, _("Switch URL...\tCTRL-S"));
 
     // Modify menu
     wxMenu *menuModif = new wxMenu;
@@ -510,7 +533,7 @@ public:
                       -1,
                       (wxObject *) NULL,
                       _("Resolve selected"),
-                      _("Remove 'conflicted' state on working copy files or directories)"));
+                      _("Remove 'conflicted' state on working copy files or directories"));
 
     toolBar->AddSeparator ();
   }
@@ -537,7 +560,7 @@ public:
                       -1,
                       (wxObject *) NULL,
                       _("Log selected"),
-                      _("Show the log messages for a set entries"));
+                      _("Show the log messages for the selected entries"));
 
     toolBar->AddSeparator ();
   }
@@ -548,6 +571,7 @@ BEGIN_EVENT_TABLE (RapidSvnFrame, wxFrame)
   EVT_MENU (ID_AddWcBookmark, RapidSvnFrame::OnAddWcBookmark)
   EVT_MENU (ID_AddRepoBookmark, RapidSvnFrame::OnAddRepoBookmark)
   EVT_MENU (ID_RemoveBookmark, RapidSvnFrame::OnRemoveBookmark)
+  EVT_MENU (ID_EditBookmark, RapidSvnFrame::OnEditBookmark)
   EVT_MENU (ID_Quit, RapidSvnFrame::OnQuit)
   EVT_MENU (ID_About, RapidSvnFrame::OnAbout)
 
@@ -564,6 +588,9 @@ BEGIN_EVENT_TABLE (RapidSvnFrame, wxFrame)
   EVT_MENU (ID_Flat, RapidSvnFrame::OnFlatView)
   EVT_MENU (ID_RefreshWithUpdate, RapidSvnFrame::OnRefreshWithUpdate)
   EVT_MENU (ID_ShowUnversioned, RapidSvnFrame::OnShowUnversioned)
+#if CHECK_SVN_SUPPORTS_EXTERNALS
+    EVT_MENU (ID_IgnoreExternals, RapidSvnFrame::OnIgnoreExternals)
+#endif
   EVT_MENU (ID_Login, RapidSvnFrame::OnLogin)
   EVT_MENU (ID_Logout, RapidSvnFrame::OnLogout)
   EVT_MENU (ID_Stop, RapidSvnFrame::OnStop)
@@ -587,15 +614,12 @@ END_EVENT_TABLE ()
 RapidSvnFrame::RapidSvnFrame (const wxString & title,
                               const wxLocale & locale)
   : wxFrame ((wxFrame *) NULL, -1, title, wxDefaultPosition, wxDefaultSize,
-             wxDEFAULT_FRAME_STYLE)
+             wxDEFAULT_FRAME_STYLE),
+    m_folder_browser (NULL), m_listCtrl (NULL), m_title (title), m_context (0),
+    m_activePane (ACTIVEPANE_FOLDER_BROWSER)
 {
   m = new Data (this, locale);
-  m_folder_browser = NULL;
-  m_listCtrl = NULL;
-  m_title = title;
   m_actionWorker = CreateActionWorker (this);
-  m_context = 0;
-  m_activePane = ACTIVEPANE_FOLDER_BROWSER;
 
   // enable trace
   wxLog::AddTraceMask (TraceMisc);
@@ -603,9 +627,14 @@ RapidSvnFrame::RapidSvnFrame (const wxString & title,
   // Retrieve a pointer to the application configuration object.
   // If the object is not created, it will be created upon the first
   // call to Get().
-  wxConfigBase *pConfig = wxConfigBase::Get ();
+  wxConfigBase *cfg = wxConfigBase::Get ();
 
-  SetIcon (wxIcon (svn_xpm));
+// @todo Would be good to have an icon bundle here
+//#ifdef __WXMSW__
+//  SetIcons (wxIconBundle (wxT("rapidsvn.exe"), 0));
+//#else
+  SetIcon (wxIcon (rapidsvn_16x16_xpm));
+//#endif
 
   // toolbar rows
   m_toolbar_rows = 1;
@@ -618,11 +647,20 @@ RapidSvnFrame::RapidSvnFrame (const wxString & title,
   m->SetRunning (false);
 
   m_horiz_splitter =
+#if wxCHECK_VERSION(2, 6, 0)
+    new wxSplitterWindow (this,
+                                    SPLITTER_WINDOW,
+                                    wxDefaultPosition,
+                                    wxDefaultSize,
+                                    SPLITTER_STYLE);
+  m_horiz_splitter->SetSashGravity(1.0f);
+#else
     new ProportionalSplitterWindow (1.0f, this,
                                     SPLITTER_WINDOW,
                                     wxDefaultPosition,
                                     wxDefaultSize,
                                     SPLITTER_STYLE);
+#endif
 
   m_info_panel = new wxPanel (m_horiz_splitter, -1,
                               wxDefaultPosition, wxDefaultSize,
@@ -644,23 +682,23 @@ RapidSvnFrame::RapidSvnFrame (const wxString & title,
                                           wxDefaultPosition,
                                           wxDefaultSize,
                                           SPLITTER_STYLE);
+#if wxCHECK_VERSION(2, 6, 0)
+  // Not a complete solution, but a workaround for now...
+  m_vert_splitter->SetSashGravity(0.2f);
+#endif
 
   // Create the list control to display files
   m_listCtrl = new FileListCtrl (m_vert_splitter, FILELIST_CTRL,
                                  wxDefaultPosition, wxDefaultSize);
 
-  m->CheckMenu (ID_Flat,              m_listCtrl->IsFlat());
+  m->CheckMenu (ID_Flat,              false);
   m->CheckMenu (ID_RefreshWithUpdate, m_listCtrl->GetWithUpdate());
   m->CheckMenu (ID_ShowUnversioned,   m_listCtrl->GetShowUnversioned());
+  if (svn::SUPPORTS_EXTERNALS)
+    m->CheckMenu (ID_IgnoreExternals,   m_listCtrl->GetIgnoreExternals());
 
   // Create the browse control
   m_folder_browser = new FolderBrowser (m_vert_splitter, FOLDER_BROWSER);
-  {
-    Preferences prefs;
-    m_folder_browser->SetListener (&m->listener);
-    m_folder_browser->SetAuthPerBookmark (prefs.authPerBookmark);
-  }
-  UpdateCurrentPath ();
 
   // Adapt the menu entries
   for (int col=0; col < FileListCtrl::COL_COUNT; col++)
@@ -681,14 +719,14 @@ RapidSvnFrame::RapidSvnFrame (const wxString & title,
   m_info_panel->SetSizer (sizer);
 
   // Read frame position
-  if (pConfig->Read (ConfigMaximized, (long int)0) == 1)
+  if (cfg->Read (ConfigMaximized, (long int)0) == 1)
     Maximize (true);
   else
   {
-    int x = pConfig->Read (ConfigLeft, 50);
-    int y = pConfig->Read (ConfigTop, 50);
-    int w = pConfig->Read (ConfigWidth, 806);
-    int h = pConfig->Read (ConfigHeight, 480);
+    int x = cfg->Read (ConfigLeft, 50);
+    int y = cfg->Read (ConfigTop, 50);
+    int w = cfg->Read (ConfigWidth, 806);
+    int h = cfg->Read (ConfigHeight, 480);
 
     Move (x, y);
     SetClientSize (w, h);
@@ -697,22 +735,30 @@ RapidSvnFrame::RapidSvnFrame (const wxString & title,
   // Get sash position for every splitter from configuration.
   int w,h;
   GetSize (&w, &h);
-  int vpos = pConfig->Read (ConfigSplitterVert, w / 3);
-  int hpos = pConfig->Read (ConfigSplitterHoriz, (3 * h) / 4);
+  int vpos = cfg->Read (ConfigSplitterVert, w / 3);
+  int hpos = cfg->Read (ConfigSplitterHoriz, (3 * h) / 4);
+
+  // initialize the folder browser
+  m_folder_browser->ReadConfig (cfg);
+  {
+    Preferences prefs;
+    m_folder_browser->SetListener        (&m->listener);
+    m_folder_browser->SetAuthCache       (prefs.useAuthCache);
+    m_folder_browser->SetAuthPerBookmark (prefs.authPerBookmark);
+  }
+  UpdateCurrentPath ();
+  UpdateFolderBrowser ();
 
   // Set sash position for every splitter.
   // Note: to not revert the order of Split calls, as the panels will be messed up.
   m_horiz_splitter->SplitHorizontally (m_info_panel, m_log, hpos);
   m_vert_splitter->SplitVertically (m_folder_browser, m_listCtrl, vpos);
-
-  // put the working copy selections in the combo browser
-  InitFolderBrowser ();
 }
 
 RapidSvnFrame::~RapidSvnFrame ()
 {
-  wxConfigBase *pConfig = wxConfigBase::Get ();
-  if (pConfig == NULL)
+  wxConfigBase *cfg = wxConfigBase::Get ();
+  if (cfg == NULL)
     return;
 
   if (m_logTracer)
@@ -722,44 +768,33 @@ RapidSvnFrame::~RapidSvnFrame ()
     delete m_actionWorker;
 
   // Save frame size and position.
-  if (IsMaximized ())
-    pConfig->Write (ConfigMaximized, 1);
-  else
+  if (!IsIconized())
   {
-    int x, y;
-    int w, h;
+    if (IsMaximized ())
+      cfg->Write (ConfigMaximized, 1);
+    else
+    {
+      int x, y;
+      int w, h;
 
-    GetClientSize (&w, &h);
-    GetPosition (&x, &y);
+      GetClientSize (&w, &h);
+      GetPosition (&x, &y);
 
-    pConfig->Write (ConfigMaximized, 0);
-    pConfig->Write (ConfigLeft, (long) x);
-    pConfig->Write (ConfigTop, (long) y);
-    pConfig->Write (ConfigWidth, (long) w);
-    pConfig->Write (ConfigHeight, (long) h);
+      cfg->Write (ConfigMaximized, 0);
+      cfg->Write (ConfigLeft, (long) x);
+      cfg->Write (ConfigTop, (long) y);
+      cfg->Write (ConfigWidth, (long) w);
+      cfg->Write (ConfigHeight, (long) h);
+    }
   }
 
-
   // Save splitter positions
-  pConfig->Write (ConfigSplitterVert,
+  cfg->Write (ConfigSplitterVert,
                   (long) m_vert_splitter->GetSashPosition ());
-  pConfig->Write (ConfigSplitterHoriz,
+  cfg->Write (ConfigSplitterHoriz,
                   (long) m_horiz_splitter->GetSashPosition ());
 
-
-  // Save the bookmarks contents
-  long item;
-  const long itemCount = m_folder_browser->GetBookmarkCount ();
-  pConfig->Write (ConfigBookmarkCount, itemCount);
-  for (item = 0; item < itemCount; item++)
-  {
-    wxString key;
-    key.Printf (ConfigBookmarkFmt, item);
-
-    const wxString & bookmark = m_folder_browser->GetBookmark (item);
-
-    pConfig->Write (key, bookmark);
-  }
+  m_folder_browser->WriteConfig (cfg);
 
   delete m;
 }
@@ -824,6 +859,14 @@ RapidSvnFrame::OnRemoveBookmark (wxCommandEvent & event)
   RemoveBookmark ();
 }
 
+
+void 
+RapidSvnFrame::OnEditBookmark (wxCommandEvent & event)
+{
+  EditBookmark ();
+}
+
+
 void
 RapidSvnFrame::OnQuit (wxCommandEvent & WXUNUSED (event))
 {
@@ -877,7 +920,7 @@ RapidSvnFrame::OnHelpStartupTips (wxCommandEvent & WXUNUSED (event))
 
 void RapidSvnFrame::OnHelp (wxCommandEvent &WXUNUSED(event))
 {
-#ifdef  USE_HTML_HELP
+#ifdef USE_HTML_HELP
     wxWindow *active = wxGetActiveWindow();
     wxString helptext;
     while (active && helptext.IsEmpty())
@@ -945,8 +988,9 @@ RapidSvnFrame::AddWcBookmark ()
       Utf8ToLocal (SVN_WC_ADM_DIR_NAME))
   {
     add = FALSE;
-    wxMessageBox (_("You cannot add a subversion "
-                    "administrative directory to the bookmarks!"),
+    wxMessageBox (
+_("You cannot add a subversion administrative \
+directory to the bookmarks!"),
                   _("Error"), wxOK);
     return;
   }
@@ -994,33 +1038,47 @@ RapidSvnFrame::RemoveBookmark ()
   }
 }
 
-void
-RapidSvnFrame::InitFolderBrowser ()
-{
-  wxConfigBase *pConfig = wxConfigBase::Get ();
 
+void
+RapidSvnFrame::EditBookmark ()
+{
   wxASSERT (m_folder_browser);
 
-  wxString key;
-  wxString bookmark;
+  const FolderItemData * bookmark = m_folder_browser->GetSelection ();
 
-  long item, count;
-  pConfig->Read (ConfigBookmarkCount, &count, 0);
-  for (item = 0; item < count; item++)
+  // if nothing is selected, or the wrong type,
+  // just exit (this should be handled by the routine
+  // that is responsible for greying out menu entries,
+  // but hey: nobody is perfect
+  if (!m_folder_browser)
+    return;
+
+  if (bookmark->getFolderType () != FOLDER_TYPE_BOOKMARK)
+    return;
+
+  // now edit the data
+  wxString oldPath (bookmark->getPath ());
+  DestinationDlg dlg (this, _("Edit Bookmark"), _("Bookmark"), 
+                      0, oldPath);
+
+  if (dlg.ShowModal () == wxID_OK)
   {
-    key.Printf (ConfigBookmarkFmt, item);
-    if (pConfig->Read (key, &bookmark, wxEmptyString))
-    {
-      m_folder_browser->AddBookmark (bookmark);
-    }
+    // and if everything goes right, remove the old
+    // selection and add the new one
+    wxString newPath (dlg.GetDestination ());
 
-    else
-      break;
+    // if nothing has changed, just exit
+    if (oldPath == newPath)
+      return;
+
+    m_folder_browser->RemoveBookmark ();
+    m_folder_browser->AddBookmark (newPath);
+    UpdateFolderBrowser ();
+    m_folder_browser->SelectBookmark (newPath);
   }
-
-  UpdateFolderBrowser ();
 }
 
+
 const svn::Targets
 RapidSvnFrame::GetActionTargets () const
 {
@@ -1031,13 +1089,12 @@ RapidSvnFrame::GetActionTargets () const
   {
     wxString path = m_folder_browser->GetPath ();
 
-    std::string pathUtf8;
-    LocalToUtf8 (path, pathUtf8);
-    if (!svn::Url::isValid (pathUtf8.c_str ()))
+    svn::Path pathUtf8 (PathUtf8 (path));
+    if (!pathUtf8.isUrl ())
     {
       wxFileName fname (path);
       path = fname.GetFullPath ();
-      LocalToUtf8 (path, pathUtf8);
+      pathUtf8 = PathUtf8 (path);
     }
 
     return svn::Targets (pathUtf8.c_str ());
@@ -1063,12 +1120,12 @@ RapidSvnFrame::GetSelectionActionFlags () const
 
     flags |= Action::IS_DIR;
     wxString path = m_folder_browser->GetPath ();
-    std::string pathUtf8;
-    LocalToUtf8 (path, pathUtf8);
-    if (!pathUtf8.empty ())
+
+    svn::Path pathUtf8 (PathUtf8 (path));
+    if (pathUtf8.length () > 0)
     {
       flags |= Action::SINGLE_TARGET;
-      if (svn::Url::isValid (pathUtf8.c_str ()))
+      if (pathUtf8.isUrl ())
       {
         flags |= Action::RESPOSITORY_TYPE;
       }
@@ -1137,6 +1194,10 @@ RapidSvnFrame::ValidateIDActionFlags (int id, unsigned int selectionActionFlags)
       baseActionFlags = ImportAction::GetBaseFlags ();
       break;
 
+    case ID_Export:
+      baseActionFlags = ExportAction::GetBaseFlags ();
+      break;
+
     case ID_Checkout:
       baseActionFlags = CheckoutAction::GetBaseFlags ();
       break;
@@ -1145,6 +1206,11 @@ RapidSvnFrame::ValidateIDActionFlags (int id, unsigned int selectionActionFlags)
       baseActionFlags = CleanupAction::GetBaseFlags ();
       break;
 
+    case ID_Lock:
+      if (svn::SUPPORTS_LOCK)
+        baseActionFlags = LockAction::GetBaseFlags ();
+      break;
+
     case ID_Log:
       baseActionFlags = LogAction::GetBaseFlags ();
       break;
@@ -1190,6 +1256,8 @@ RapidSvnFrame::ValidateIDActionFlags (int id, unsigned int selectionActionFlags)
       break;
 
     case ID_Diff:
+    case ID_DiffBase:
+    case ID_DiffHead:
       baseActionFlags = DiffAction::GetBaseFlags ();
       break;
 
@@ -1198,6 +1266,11 @@ RapidSvnFrame::ValidateIDActionFlags (int id, unsigned int selectionActionFlags)
       baseActionFlags = Action::SINGLE_TARGET|Action::MULTIPLE_TARGETS|Action::RESPOSITORY_TYPE|Action::VERSIONED_WC_TYPE|Action::UNVERSIONED_WC_TYPE;
       break;
 
+    case ID_Unlock:
+      if (svn::SUPPORTS_LOCK)
+        baseActionFlags = UnlockAction::GetBaseFlags ();
+      break;
+
     case ID_Edit:
       baseActionFlags = ViewAction::GetEditFlags ();
       break;
@@ -1319,6 +1392,10 @@ RapidSvnFrame::OnFileCommand (wxCommandEvent & event)
       action = new ImportAction (this);
       break;
 
+    case ID_Export:
+      action = new ExportAction (this);
+      break;
+
     case ID_Checkout:
       action = new CheckoutAction (this);
       break;
@@ -1327,6 +1404,10 @@ RapidSvnFrame::OnFileCommand (wxCommandEvent & event)
       action = new CleanupAction (this);
       break;
 
+    case ID_Lock:
+      action = new LockAction (this);
+      break;
+
     case ID_Log:
       action = new LogAction (this);
       break;
@@ -1389,19 +1470,23 @@ RapidSvnFrame::OnFileCommand (wxCommandEvent & event)
         break;
       }
 
+    case ID_Unlock:
+      action = new UnlockAction (this);
+      break;
+
     case ID_Edit:
       action = new ViewAction (this);
       break;
 
     case ID_CreateRepository:
-      wxMessageBox (
-        _("Please use the command line utility 'svnadmin'\n"
-          "to create a new repository.\n\n"
-          "This command line utility is not part of the\n"
-          "RapidSVN distribution.\n\n"
-          "More information about subversion:\n"
-          "http://svnbook.red-bean.com/\n"
-          "http://subversion.tigris.org"),
+      wxMessageBox ( 
+_("Please use the command line utility 'svnadmin'\n\
+to create a new repository.\n\n\
+This command line utility is not part of the\n\
+RapidSVN distribution.\n\n\
+More information about subversion:\n\
+http://svnbook.red-bean.com/\n\
+http://subversion.tigris.org"),
         _("Information"),
         wxOK);
       break;
@@ -1451,7 +1536,7 @@ RapidSvnFrame::OnActionEvent (wxCommandEvent & event)
       }
       else
       {
-        Trace (_("Error receiving action event!"));
+        Trace (_("Internal Error: no client data for action event!"));
       }
 
       if ((actionFlags & Action::UPDATE_LATER) != 0)
@@ -1575,7 +1660,11 @@ RapidSvnFrame::OnActionEvent (wxCommandEvent & event)
 void
 RapidSvnFrame::ShowInfo ()
 {
-  FileInfo fileInfo (m_context);
+  bool withUpdate = false;
+  if (m_listCtrl)
+    withUpdate = m_listCtrl->GetWithUpdate ();
+
+  FileInfo fileInfo (m_context, withUpdate);
 
   try
   {
@@ -1604,9 +1693,28 @@ RapidSvnFrame::ShowInfo ()
 void
 RapidSvnFrame::OnFolderBrowserSelChanged (wxTreeEvent & event)
 {
-  m_activePane = ACTIVEPANE_FOLDER_BROWSER;
-  UpdateCurrentPath ();
-  UpdateFileList ();
+  try
+  {
+    m_activePane = ACTIVEPANE_FOLDER_BROWSER;
+
+    // Update the menu and list control flat-mode setting 
+    bool flatMode = m_folder_browser->IsFlat ();
+    m_listCtrl->SetFlat (flatMode);
+    m->CheckMenu (ID_Flat, flatMode);
+    
+    // Disable menu entry if no path is selected (e.g. root)
+    const wxString & path = m_folder_browser->GetPath ();  
+    m->MenuBar->Enable (ID_Flat, !path.IsEmpty ());
+
+    UpdateCurrentPath ();
+    UpdateFileList ();
+
+    m_folder_browser->ExpandSelection ();
+  }
+  catch(...)
+  {
+    Trace (_("Exception occured during filelist update"));
+  }
 }
 
 void
@@ -1633,6 +1741,7 @@ RapidSvnFrame::ShowPreferences ()
 
   if (ok)
   {
+    m_folder_browser->SetAuthCache (prefs.useAuthCache);
     m_folder_browser->SetAuthPerBookmark (prefs.authPerBookmark);
   }
 }
@@ -1651,7 +1760,10 @@ RapidSvnFrame::UpdateCurrentPath ()
     m_context = m_folder_browser->GetContext ();
   }
 
-  SetTitle (m_title + wxT(": ") + m_currentPath);
+  if (m_currentPath.Length () > 0)
+    SetTitle (m_title + wxT(" - ") + m_currentPath);
+  else
+    SetTitle (m_title);
 }
 
 bool
@@ -1689,14 +1801,10 @@ RapidSvnFrame::Perform (Action * action)
     m->SetRunning (true);
     m->listener.cancel (false);
 
-    std::string currentPathUtf8;
-    LocalToUtf8(m_currentPath, currentPathUtf8);
-    action->SetPath (svn::Path (currentPathUtf8));
+    svn::Path currentPathUtf8 (PathUtf8 (m_currentPath));
+    action->SetPath (currentPathUtf8);
     action->SetContext (m_context);
-    if ((action->GetFlags () & Action::WITHOUT_TARGET) == 0)
-    {
-      action->SetTargets (GetActionTargets ());
-    }
+    action->SetTargets (GetActionTargets ());
     action->SetTracer (m_logTracer, false);
     m_actionWorker->SetTracer (m_logTracer);
     m_actionWorker->SetContext (m_context, false);
@@ -1731,12 +1839,20 @@ RapidSvnFrame::OnColumn (wxCommandEvent & event)
   UpdateFileList ();
 }
 
+
 void
 RapidSvnFrame::OnFlatView (wxCommandEvent & event)
 {
-  bool checked = m->IsMenuChecked (ID_Flat);
-  //m->CheckMenu (ID_Flat, !checked);
-  m_listCtrl->SetFlat (checked);
+  bool newFlatMode = !m_folder_browser->IsFlat ();
+
+  // if this cannot be set (e.g. invalid selecttion
+  // like the root element, we uncheck this
+  if (!m_folder_browser->SetFlat (newFlatMode))
+    newFlatMode = false;
+
+  m->CheckMenu (ID_Flat, newFlatMode);
+  m_listCtrl->SetFlat (newFlatMode);
+    
   UpdateFileList ();
 }
 
@@ -1758,6 +1874,16 @@ RapidSvnFrame::OnShowUnversioned (wxCommandEvent & WXUNUSED (event))
   UpdateFolderBrowser ();
 }
 
+void
+RapidSvnFrame::OnIgnoreExternals (wxCommandEvent & WXUNUSED (event))
+{
+  if (svn::SUPPORTS_EXTERNALS)
+  {
+    bool checked = m->IsMenuChecked (ID_IgnoreExternals);
+    m_listCtrl->SetIgnoreExternals (checked);
+  }
+  UpdateFolderBrowser ();
+}
 
 void
 RapidSvnFrame::OnColumnReset (wxCommandEvent &)
