@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library (in the file LGPL.txt); if not, 
+ * write to the Free Software Foundation, Inc., 51 Franklin St, 
+ * Fifth Floor, Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -45,6 +57,8 @@ namespace svn
   {
     Pool pool;
 
+    m_pathIsUrl = false;
+
     if (path == 0)
       m_path = "";
     else
@@ -53,6 +67,12 @@ namespace svn
         svn_path_internal_style (path, pool.pool () );
     
       m_path = int_path;
+      
+      if (svn::Url::isValid (int_path))
+      {
+        m_pathIsUrl = true;
+        m_path = svn::Url::escape (int_path);
+      }
     }
   }
 
@@ -69,11 +89,13 @@ namespace svn
   }
 
   Path& 
-  Path::operator=(const Path & path)
+  Path::operator= (const Path & path)
   {
     if (this == &path)
       return *this;
-    m_path = path.c_str();
+
+    init (path.c_str ());
+
     return *this;
   }
 
@@ -83,6 +105,12 @@ namespace svn
     return m_path.length () > 0;
   }
 
+  const bool
+  Path::isUrl () const
+  {
+    return m_pathIsUrl;
+  }
+
   void
   Path::addComponent (const char * component)
   {
@@ -258,8 +286,7 @@ end:
     const char * tempdir = NULL;
     Pool pool;
 
-    apr_status_t stat = apr_temp_dir_get (&tempdir, pool);
-    if (!APR_STATUS_IS_SUCCESS(stat))
+    if (apr_temp_dir_get (&tempdir, pool) != APR_SUCCESS)
     {
       tempdir = NULL;
     }
