@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -24,7 +36,7 @@
 #include "utils.hpp"
 
 SwitchAction::SwitchAction (wxWindow * parent)
- : Action (parent, _("Switch"), GetBaseFlags ())
+ : Action (parent, _("Switch URL"), GetBaseFlags ())
 {
 }
 
@@ -48,7 +60,7 @@ SwitchAction::Prepare ()
   // create flags for the dialog
   int flags = UpdateDlg::WITH_URL;
 
-  UpdateDlg dlg (GetParent (), _("Switch"), flags,
+  UpdateDlg dlg (GetParent (), _("Switch URL"), flags,
                  true);
   dlg.GetData ().url = url;
 
@@ -66,7 +78,7 @@ bool
 SwitchAction::Perform ()
 {
   svn::Path path = GetTarget ();
-  std::string urlUtf8 (LocalToUtf8 (m_data.url));
+  svn::Path urlUtf8 (PathUtf8 (m_data.url));
   svn::Revision revision (svn::Revision::HEAD);
 
   if (!m_data.useLatest)
