@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -16,9 +28,11 @@
 #include "wx/valgen.h"
 
 // svncpp
-#include "svncpp/url.hpp"
+#include "svncpp/path.hpp"
 
 // app
+#include "hist_entries.hpp"
+#include "hist_val.hpp"
 #include "update_dlg.hpp"
 #include "update_data.hpp"
 #include "utils.hpp"
@@ -35,14 +49,14 @@ struct UpdateDlg::Data
 private:
   wxTextCtrl * m_textRevision;
   wxCheckBox * m_checkUseLatest;
-  wxTextCtrl * m_textUrl;
+  wxComboBox * m_comboUrl;
   wxButton * m_buttonOk;
   const int m_flags;
 public:
   UpdateData data;
 
   Data (wxWindow * window, int flags, bool recursive)
-    : m_textRevision (0), m_checkUseLatest (0), m_textUrl (0),
+    : m_textRevision (0), m_checkUseLatest (0), m_comboUrl (0),
       m_buttonOk (0), m_flags (flags)
   {
     data.recursive = recursive;
@@ -57,11 +71,11 @@ public:
       wxStaticBox * box = new wxStaticBox (window, -1, _("URL"));
       wxStaticBoxSizer * sizer =
         new wxStaticBoxSizer (box, wxHORIZONTAL);
-      wxTextValidator val (wxFILTER_NONE, &data.url);
-      m_textUrl = new wxTextCtrl (window, ID_URL, wxEmptyString,
-                                  wxDefaultPosition,
-                                  wxDefaultSize, 0, val);
-      sizer->Add (m_textUrl, 1, wxALL | wxEXPAND, 5);
+      HistoryValidator val (HISTORY_REPOSITORY, &data.url);
+      m_comboUrl =
+        new wxComboBox (window, ID_URL, wxEmptyString, wxDefaultPosition,
+                        wxSize (235, -1), 0, 0, wxCB_DROPDOWN, val);
+      sizer->Add (m_comboUrl, 1, wxALL | wxEXPAND, 5);
       middleSizer->Add (sizer, 1, wxALL | wxEXPAND, 5);
     }
 
@@ -140,9 +154,8 @@ public:
     }
     if (ok && withUrl ())
     {
-      std::string UrlUtf8;
-      LocalToUtf8 (m_textUrl->GetValue (), UrlUtf8);
-      ok = svn::Url::isValid (UrlUtf8.c_str ());
+      svn::Path UrlUtf8 (PathUtf8 (m_comboUrl->GetValue ()));
+      ok = UrlUtf8.isUrl ();
     }
 
     m_buttonOk->Enable (ok);
