@@ -1,9 +1,21 @@
 /*
  * ====================================================================
- * Copyright (c) 2002-2004 The RapidSvn Group.  All rights reserved.
+ * Copyright (c) 2002-2006 The RapidSvn Group.  All rights reserved.
  *
- * This software is licensed as described in the file LICENSE.txt,
- * which you should have received as part of this distribution.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in the file GPL.txt); if not, write to 
+ * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  *
  * This software consists of voluntary contributions made by many
  * individuals.  For exact contribution history, see the revision
@@ -60,9 +72,9 @@ ViewAction::Perform ()
     path = Utf8ToLocal (GetTarget ().c_str ());
   else
   {
-    std::string pathUtf8 (LocalToUtf8 (m_data.path));
+    svn::Path pathUtf8 (PathUtf8 (m_data.path));
     path = Utf8ToLocal  (
-        GetPathAsTempFile(svn::Path (pathUtf8.c_str ()),
+        GetPathAsTempFile(pathUtf8.c_str (),
             m_data.revision).c_str ());
   }
 
